package org.livee.data.batch;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.livee.data.expr.AbstractData;
import org.livee.data.tools.LogManager;
import org.livee.data.tools.NeighborManager;


/**
 * @author Hideya Ochiai, The University of Tokyo
 * 
 */
public class ChildProfileMergeBatch implements AbstractBatch {

	int m_Interval;
	
	public ChildProfileMergeBatch(){
		org.livee.data.tools.ConfigManager config=org.livee.data.tools.ConfigManager.getInstance();
		m_Interval=config.getIntParameter("/c:liveeConfiguration/c:master/c:childProfileMerger/c:interval/text()");
		if(m_Interval<=0){
			m_Interval=3600;
		}
	}
	
	public int interval(){
		return m_Interval;
	}
	
	public void execute() throws Exception {
		
		LogManager.postNormal("master.childProfileMerger.begin", null);
		
		NeighborManager neighbor=NeighborManager.getInstance();
		AbstractData[] adminSites=neighbor.getChildrenAdminSite();
		
		if(adminSites!=null){
			for(int i=0;i<adminSites.length;i++){
				AbstractData adminSite=adminSites[i];

				String site=adminSite.getAttribute("name");
				java.util.Iterator itr=adminSite.getChildren().iterator();
				boolean b_succeeded=false;
				while(itr.hasNext() && !b_succeeded){
					AbstractData adminServer=(AbstractData)itr.next();
					String service=adminServer.getAttribute("service");
					String url=adminServer.getAttribute("url");
					String linkStatus=adminServer.getAttribute("linkStatus");
					
					if(service.equals("Admin200703") && (linkStatus==null || linkStatus.equals("up"))){
						String profileAll=null;
						try{
							org.livee.data.soap.client.Admin200703Service srv=new org.livee.data.soap.client.Admin200703ServiceLocator();
							org.livee.data.soap.client.Admin200703_PortType admin=srv.getAdmin200703(new java.net.URL(url));
							profileAll=admin.getProfileAll();
						}
						catch(Exception e){
							// Warning
							LogManager.postWarning("master.childProfileMerger.warning", "Service access failure: "+e.getMessage());
							continue ;
						}
											
						if(profileAll!=null){
							// 1. Try to parse the latestDataAll;
							org.w3c.dom.Document doc=org.livee.util.Xml.parseXml(profileAll);
							if(doc==null){
								// Warning throw new Exception("An error occured while loading and parsing the data");
								LogManager.postWarning("master.childProfileMerger.warning", "XML parse failure");
								continue ;
							}
							
							org.livee.data.expr.AbstractData sensorCollection=null;
							try{
								org.w3c.dom.Element xml=doc.getDocumentElement();	
								org.livee.data.expr.ParserFactory pf=org.livee.data.expr.ParserFactory.getInstance();
								org.livee.data.expr.AbstractParser sensorCollection_parser=pf.get("collection");
								
								sensorCollection=sensorCollection_parser.parse(xml);
							}
							catch(Exception e){
								// e.printStackTrace();
								// Warning 
								LogManager.postWarning("master.childProfileMerger.warning", "While parsing the data: "+e.getMessage());
								continue ;
							}
							
							// 2. Save the gotton data;
							try{
								org.livee.data.db.SetAdminProfileCommand command=org.livee.data.db.CommandFactory.createSetAdminProfileCommand(site,sensorCollection);
								org.livee.data.db.DatabaseAdapter db=org.livee.data.db.DatabaseAdapter.getInstance();
								db.postCommand(command);
								
								if(!command.succeeded()){
									// Warning throw new Exception("Failed to save the got data");
									LogManager.postWarning("master.childProfileMerger.warning", "While saving the data");
									continue ;
								}
								b_succeeded=true;
							}
							catch(Exception e){
								// e.printStackTrace();
								// Warning 
								LogManager.postWarning("master.childProfileMerger.warning", "While saving the data: "+e.getMessage());
								continue ;
							}

						}
					}
					if(!b_succeeded){
						// Error (failure of accessing site ); 
						//site;
						LogManager.postError("master.childProfileMerger.error", "Site access failure: "+site);
					}
				}
			}
		}
		LogManager.postNormal("master.childProfileMerger.end", null);
	}

	public String getBatchName() {
		return "ChildProfileMerge";
	}

}
