package org.livee.data.db;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.util.W3CTimestamp;
import java.util.Calendar;

import org.livee.data.tools.LogManager;


/**
 * @author Hideya Ochiai, The University of Tokyo
 * 
 */
public class AggregationCommand implements IDbCommand {

	boolean m_bSucceeded;
	boolean m_bTryNext;
	
	protected AggregationCommand(){
		m_bSucceeded=false;
		m_bTryNext=false;
	}
	
	public boolean succeeded(){
		return m_bSucceeded;
	}
	
	public boolean tryNext(){
		return m_bTryNext;
	}
	
	public void execCommand(Connection connection, Map conf) throws Exception {

		java.sql.Statement stmt=null;
		org.livee.data.tools.ConfigManager config=org.livee.data.tools.ConfigManager.getInstance();
		
		try{
			stmt=connection.createStatement();
			
			stmt.execute("START TRANSACTION;");
			
			// Read Agg Profile
			java.util.Calendar comparator=java.util.Calendar.getInstance();
			java.util.Calendar aggDoneHour=null;
			java.util.Calendar aggDoneDay=null;
			java.util.Calendar aggDoneMonth=null;
			java.sql.ResultSet rset=stmt.executeQuery("SELECT scope, done FROM aggregation.profile;");
			while(rset.next()){
				String scope=rset.getString("scope");
				java.sql.Timestamp done=rset.getTimestamp("done");
				String done_string=W3CTimestamp.toString(done, java.util.TimeZone.getDefault());
	
				if(scope.equals("hour")){
					aggDoneHour=W3CTimestamp.parse(done_string);
				}else if(scope.equals("day")){
					aggDoneDay=W3CTimestamp.parse(done_string);
				}else if(scope.equals("month")){
					aggDoneMonth=W3CTimestamp.parse(done_string);
				}
			}
			rset.close();
			
			// Set to default setting if not constructed..
			if(aggDoneHour==null){
				String default_startup=config.getStringParameter("/c:liveeConfiguration/c:master/c:aggregator/c:start/text()");
				aggDoneHour=W3CTimestamp.parse(default_startup);
			}
			if(aggDoneDay==null){
				String default_startup=config.getStringParameter("/c:liveeConfiguration/c:master/c:aggregator/c:start/text()");
				aggDoneDay=W3CTimestamp.parse(default_startup);
			}
			if(aggDoneMonth==null){
				String default_startup=config.getStringParameter("/c:liveeConfiguration/c:master/c:aggregator/c:start/text()");
				aggDoneMonth=W3CTimestamp.parse(default_startup);
			}
			
			// load configuration of hour aggregator
			int time=config.getIntParameter("/c:liveeConfiguration/c:master/c:aggregator/c:hourAggregator/c:thresholdTime/text()");
			String strTimeBase=config.getStringParameter("/c:liveeConfiguration/c:master/c:aggregator/c:hourAggregator/c:thresholdTimeBase/text()");
			int timeBase=Calendar.MINUTE;
			if(strTimeBase.equals("second")){ timeBase=Calendar.SECOND; }
			else if(strTimeBase.equals("hour")){ timeBase=Calendar.HOUR; }
			else if(strTimeBase.equals("day")){ timeBase=Calendar.DATE; }
			else if(strTimeBase.equals("month")){ timeBase=Calendar.MONTH; }
			if(time<=0){ time=70; }
			
			// aggregation of work => hour
			m_bTryNext=false;
			comparator.add(timeBase, -time); 
			if(aggDoneHour.before(comparator)){
				m_bTryNext=true;
				aggDoneHour.set(Calendar.MINUTE,0);
				aggDoneHour.set(Calendar.SECOND,0);
				aggDoneHour.set(Calendar.MILLISECOND,0);
							
				Calendar from=aggDoneHour;
				Calendar to=Calendar.getInstance();
				to.setTimeInMillis(from.getTimeInMillis());
				to.add(Calendar.HOUR, +1);
				
				aggregationOfWork(stmt,from,to);
			}
			

			// load configuration of day aggregator
			time=config.getIntParameter("/c:liveeConfiguration/c:master/c:aggregator/c:dayAggregator/c:thresholdTime/text()");
			strTimeBase=config.getStringParameter("/c:liveeConfiguration/c:master/c:aggregator/c:dayAggregator/c:thresholdTimeBase/text()");
			timeBase=Calendar.DAY_OF_MONTH;
			if(strTimeBase.equals("second")){ timeBase=Calendar.SECOND; }
			else if(strTimeBase.equals("minute")){ timeBase=Calendar.MINUTE; }
			else if(strTimeBase.equals("hour")){ timeBase=Calendar.HOUR; }
			else if(strTimeBase.equals("month")){ timeBase=Calendar.MONTH; }
			if(time<=0){ time=1; }

			// aggregation of hour => day
			comparator=aggDoneHour;
			comparator.add(timeBase, -time);
			if(aggDoneDay.before(comparator)){
				aggDoneDay.set(Calendar.HOUR_OF_DAY,0);
				aggDoneDay.set(Calendar.MINUTE,0);
				aggDoneDay.set(Calendar.SECOND,0);
				aggDoneDay.set(Calendar.MILLISECOND,0);
				Calendar from=aggDoneDay;
				Calendar to=Calendar.getInstance();
				to.setTimeInMillis(from.getTimeInMillis());
				to.add(Calendar.DAY_OF_MONTH, +1);
								
				aggregationOfHour(stmt,from,to);
			}
			
			

			// load configuration of month aggregator
			time=config.getIntParameter("/c:liveeConfiguration/c:master/c:aggregator/c:monthAggregator/c:thresholdTime/text()");
			strTimeBase=config.getStringParameter("/c:liveeConfiguration/c:master/c:aggregator/c:monthAggregator/c:thresholdTimeBase/text()");
			timeBase=Calendar.MONTH;
			if(strTimeBase.equals("second")){ timeBase=Calendar.SECOND; }
			else if(strTimeBase.equals("minute")){ timeBase=Calendar.MINUTE; }
			else if(strTimeBase.equals("hour")){ timeBase=Calendar.HOUR; }
			else if(strTimeBase.equals("day")){ timeBase=Calendar.DATE; }
			if(time<=0){ time=1; }

			// aggregation of day => month
			comparator=aggDoneDay;
			comparator.add(timeBase, -time);
			if(aggDoneMonth.before(comparator)){
				aggDoneMonth.set(Calendar.DAY_OF_MONTH,1);
				aggDoneMonth.set(Calendar.HOUR_OF_DAY,0);
				aggDoneMonth.set(Calendar.MINUTE,0);
				aggDoneMonth.set(Calendar.SECOND,0);
				aggDoneMonth.set(Calendar.MILLISECOND,0);
				Calendar from=aggDoneMonth;
				Calendar to=Calendar.getInstance();
				to.setTimeInMillis(from.getTimeInMillis());
				to.add(Calendar.MONTH, +1);
				
				aggregationOfDay(stmt,from,to);
			}
		
			stmt.execute("COMMIT;");
			m_bSucceeded=true;
		}
		catch(Exception e){
			// e.printStackTrace();
			stmt.execute("ROLLBACK;");
			throw e;
		}
		finally{
			stmt.close();
		}
	}
	
	private void aggregationOfWork(java.sql.Statement stmt, Calendar from, Calendar to) throws java.sql.SQLException {
		// aggregation of work [from,to] => hour(from)
		String str_from=from.getTime().toString();
		String str_to=to.getTime().toString();
		
		LogManager.postNormal("master.aggregator.hourAggregator.begin", str_from);
		
		java.util.ArrayList sqls=new java.util.ArrayList();
		
		String sql="SELECT id,SUM(value),AVG(value),MAX(value),MIN(value),COUNT(value) FROM aggregation.work WHERE time>=\'"+str_from+"\' AND time<\'"+str_to+"\' GROUP BY id;";
		java.sql.ResultSet rset=stmt.executeQuery(sql);
		while(rset.next()){
			String id=rset.getString(1);
			String sum=rset.getString(2);
			String avg=rset.getString(3);
			String max=rset.getString(4);
			String min=rset.getString(5);
			String count=rset.getString(6);
			String aggSQL_Base="INSERT INTO aggregation.hour (id,time,attrs,aggregationType,value) VALUES (\'"+id+"\',\'"+str_from+"\',\'{}\',";			
			sqls.add(aggSQL_Base+"\'sum\',"+sum+");");
			sqls.add(aggSQL_Base+"\'avg\',"+avg+");");
			sqls.add(aggSQL_Base+"\'max\',"+max+");");
			sqls.add(aggSQL_Base+"\'min\',"+min+");");
			sqls.add(aggSQL_Base+"\'count\',"+count+");");
		}
		rset.close();

		java.util.Iterator itr=sqls.iterator();
		while(itr.hasNext()){
			String aggSQL=(String)itr.next();
			stmt.execute(aggSQL);
		}

		// deletion of work (,to)
		sql="DELETE FROM aggregation.work WHERE time<\'"+str_to+"\';";
		try{ stmt.execute(sql); } catch(Exception e){e.printStackTrace(); } 
		// profile change (to)
		sql="UPDATE aggregation.profile SET done=\'"+str_to+"\' WHERE scope=\'hour\';";
		if(stmt.executeUpdate(sql)==0){
			sql="INSERT INTO aggregation.profile (scope,done) VALUES (\'hour\',\'"+str_to+"\');";
			stmt.execute(sql);
		}

		LogManager.postNormal("master.aggregator.hourAggregator.end", str_from);
		
	}
	
	private void aggregationOfHour(java.sql.Statement stmt, Calendar from, Calendar to) throws java.sql.SQLException {

		// aggregation of hour [from,to] => day(from) 
		String str_from=from.getTime().toString();
		String str_to=to.getTime().toString();
		
		LogManager.postNormal("master.aggregator.dayAggregator.begin",str_from);

		String sql=null;
		java.sql.ResultSet rset=null;
		String aggSQL_Base=null;
		java.util.ArrayList sqls=new java.util.ArrayList();

		// sum
		sql="SELECT id,SUM(value) FROM aggregation.hour WHERE aggregationType=\'sum\' AND time>=\'"+str_from+"\' AND time<\'"+str_to+"\' GROUP BY id;";
		rset=stmt.executeQuery(sql);
		aggSQL_Base="INSERT INTO aggregation.day (aggregationType,time,attrs,id,value) VALUES (\'sum\',\'"+str_from+"\',\'{}\',";
		while(rset.next()){
			String id=rset.getString(1);
			String sum=rset.getString(2);
			String aggSQL=aggSQL_Base+"\'"+id+"\',"+sum+");";
			sqls.add(aggSQL);
		}
		rset.close();
		
		// avg
		sql="SELECT id,AVG(value) FROM aggregation.hour WHERE aggregationType=\'avg\' AND time>=\'"+str_from+"\' AND time<\'"+str_to+"\' GROUP BY id;";
		rset=stmt.executeQuery(sql);
		aggSQL_Base="INSERT INTO aggregation.day (aggregationType,time,attrs,id,value) VALUES (\'avg\',\'"+str_from+"\',\'{}\',";
		while(rset.next()){
			String id=rset.getString(1);
			String avg=rset.getString(2);
			String aggSQL=aggSQL_Base+"\'"+id+"\',"+avg+");";
			sqls.add(aggSQL);
		}
		rset.close();
		
		// max
		sql="SELECT id,MAX(value) FROM aggregation.hour WHERE aggregationType=\'max\' AND time>=\'"+str_from+"\' AND time<\'"+str_to+"\' GROUP BY id;";
		rset=stmt.executeQuery(sql);
		aggSQL_Base="INSERT INTO aggregation.day (aggregationType,time,attrs,id,value) VALUES (\'max\',\'"+from.getTime().toString()+"\',\'{}\',";
		while(rset.next()){
			String id=rset.getString(1);
			String max=rset.getString(2);
			sqls.add(aggSQL_Base+"\'"+id+"\',"+max+");");
		}
		rset.close();
		
		// min
		sql="SELECT id,MIN(value) FROM aggregation.hour WHERE aggregationType=\'min\' AND time>=\'"+str_from+"\' AND time<\'"+str_to+"\' GROUP BY id;";
		rset=stmt.executeQuery(sql);
		aggSQL_Base="INSERT INTO aggregation.day (aggregationType,time,attrs,id,value) VALUES (\'min\',\'"+from.getTime().toString()+"\',\'{}\',";
		while(rset.next()){
			String id=rset.getString(1);
			String min=rset.getString(2);
			sqls.add(aggSQL_Base+"\'"+id+"\',"+min+");");
		}
		rset.close();

		// count
		sql="SELECT id,SUM(value) FROM aggregation.hour WHERE aggregationType=\'count\' AND time>=\'"+str_from+"\' AND time<\'"+str_to+"\' GROUP BY id;";
		rset=stmt.executeQuery(sql);
		aggSQL_Base="INSERT INTO aggregation.day (aggregationType,time,attrs,id,value) VALUES (\'count\',\'"+str_from+"\',\'{}\',";
		while(rset.next()){
			String id=rset.getString(1);
			String count=rset.getString(2);
			sqls.add(aggSQL_Base+"\'"+id+"\',"+count+");");
		}
		rset.close();

		java.util.Iterator itr=sqls.iterator();
		while(itr.hasNext()){
			String aggSQL=(String)itr.next();
			stmt.execute(aggSQL);
		}
		
		// profile change (to)
		sql="UPDATE aggregation.profile SET done=\'"+str_to+"\' WHERE scope=\'day\';";
		if(stmt.executeUpdate(sql)==0){
			sql="INSERT INTO aggregation.profile (scope,done) VALUES (\'day\',\'"+str_to+"\');";
			stmt.execute(sql);
		}

		LogManager.postNormal("master.aggregator.dayAggregator.end",str_from);
	}

	private void aggregationOfDay(java.sql.Statement stmt, Calendar from, Calendar to) throws java.sql.SQLException {
		
		// aggregation of day [from,to] => month(from) 
		String str_from=from.getTime().toString();
		String str_to=to.getTime().toString();
		
		LogManager.postNormal("master.aggregator.monthAggregator.begin",str_from);

		String sql=null;
		java.sql.ResultSet rset=null;
		String aggSQL_Base=null;
		java.util.ArrayList sqls=new java.util.ArrayList();
		// sum
		sql="SELECT id,SUM(value) FROM aggregation.day WHERE aggregationType=\'sum\' AND time>=\'"+str_from+"\' AND time<\'"+str_to+"\' GROUP BY id;";
		rset=stmt.executeQuery(sql);
		aggSQL_Base="INSERT INTO aggregation.month (aggregationType,time,attrs,id,value) VALUES (\'sum\',\'"+str_from+"\',\'{}\',";
		while(rset.next()){
			String id=rset.getString(1);
			String sum=rset.getString(2);
			sqls.add(aggSQL_Base+"\'"+id+"\',"+sum+");");
		}
		rset.close();
		
		// avg
		sql="SELECT id,AVG(value) FROM aggregation.day WHERE aggregationType=\'avg\' AND time>=\'"+str_from+"\' AND time<\'"+str_to+"\' GROUP BY id;";
		rset=stmt.executeQuery(sql);
		aggSQL_Base="INSERT INTO aggregation.month (aggregationType,time,attrs,id,value) VALUES (\'avg\',\'"+str_from+"\',\'{}\',";
		while(rset.next()){
			String id=rset.getString(1);
			String avg=rset.getString(2);
			sqls.add(aggSQL_Base+"\'"+id+"\',"+avg+");");
		}
		rset.close();

		// max
		sql="SELECT id,MAX(value) FROM aggregation.day WHERE aggregationType=\'max\' AND time>=\'"+str_from+"\' AND time<\'"+str_to+"\' GROUP BY id;";
		rset=stmt.executeQuery(sql);
		aggSQL_Base="INSERT INTO aggregation.month (aggregationType,time,attrs,id,value) VALUES (\'max\',\'"+str_from+"\',\'{}\',";
		while(rset.next()){
			String id=rset.getString(1);
			String max=rset.getString(2);
			sqls.add(aggSQL_Base+"\'"+id+"\',"+max+");");
		}
		rset.close();

		// min
		sql="SELECT id,MIN(value) FROM aggregation.day WHERE aggregationType=\'min\' AND time>=\'"+str_from+"\' AND time<\'"+str_to+"\' GROUP BY id;";
		rset=stmt.executeQuery(sql);
		aggSQL_Base="INSERT INTO aggregation.month (aggregationType,time,attrs,id,value) VALUES (\'min\',\'"+str_from+"\',\'{}\',";
		while(rset.next()){
			String id=rset.getString(1);
			String min=rset.getString(2);
			sqls.add(aggSQL_Base+"\'"+id+"\',"+min+");");
		}
		rset.close();

		// count
		sql="SELECT id,SUM(value) FROM aggregation.day WHERE aggregationType=\'count\' AND time>=\'"+str_from+"\' AND time<\'"+str_to+"\' GROUP BY id;";
		rset=stmt.executeQuery(sql);
		aggSQL_Base="INSERT INTO aggregation.month (aggregationType,time,attrs,id,value) VALUES (\'count\',\'"+str_from+"\',\'{}\',";
		while(rset.next()){
			String id=rset.getString(1);
			String count=rset.getString(2);
			sqls.add(aggSQL_Base+"\'"+id+"\',"+count+");");
		}
		rset.close();

		java.util.Iterator itr=sqls.iterator();
		while(itr.hasNext()){
			String aggSQL=(String)itr.next();
			stmt.execute(aggSQL);
		}

		// profile change (to)
		sql="UPDATE aggregation.profile SET done=\'"+str_to+"\' WHERE scope=\'month\';";
		if(stmt.executeUpdate(sql)==0){
			sql="INSERT INTO aggregation.profile (scope,done) VALUES (\'month\',\'"+str_to+"\');";
			stmt.execute(sql);
		}
		LogManager.postNormal("master.aggregator.monthAggregator.end",str_from);
	}

	public String getCommandName() {
		return "Aggregation";
	}

	public short getImplVersion() {
		return 0x0100;
	}

}
