package org.livee.data.db;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.sql.DriverManager;
import java.util.Map;

/**
 * Database Adapter for "Live E! DataManager Version 1.0" (datamanager200703.schema.db)
 * @author Hideya Ochiai
 * @version 1.0
 */
public class DatabaseAdapter {

	static DatabaseAdapter m_DatabaseAdapter=null;
	
	/**
	 * Connection
	 */
	static String m_ConnectionString="jdbc:postgresql:datamanager";
	//static String m_ConnectionString="jdbc:postgresql://live-e-develop.hongo.wide.ad.jp/datamanager";
	static String m_ConnectionUser="postgres";
	static String m_ConnectionPassword="postgres";
	Connection m_SqlConnection;
	
	/**
	 * Configuration
	 * 
	 * 
	 * Version
	 */
	Map m_DbConfiguration; 
	
	
	public static DatabaseAdapter getInstance(){
		
		try{
			if(m_DatabaseAdapter==null){
				new DatabaseAdapter();
			}
			return m_DatabaseAdapter;
		}
		catch(Exception e){
			e.printStackTrace();
		}
		return null;
	}
	
	/*
	 * (Non javadoc)
	 * Only the create method can create DatabaseAdapter directly
	 */
	private DatabaseAdapter() throws Exception {
		
		m_DbConfiguration=new java.util.Hashtable();
		
		org.livee.data.tools.ConfigManager config=org.livee.data.tools.ConfigManager.getInstance();
		String connection=config.getStringParameter("/c:liveeConfiguration/c:dbAccess/text()");
		
		// JDBC Connector
		Class.forName("org.postgresql.Driver");
		if(connection!=null && connection.contains(":")){
			m_ConnectionString=connection;
		}
		
		initConnection();
	}
	
	private void initConnection() throws Exception {
		
		m_SqlConnection=DriverManager.getConnection(m_ConnectionString,m_ConnectionUser,m_ConnectionPassword);
	    
		/**
		 * Read the db configuration into a bag of "configuration"
		 */
		try{
			java.sql.Statement stmt=m_SqlConnection.createStatement();
			java.sql.ResultSet rset=stmt.executeQuery("SELECT name,value FROM conf;");
		    	
		    while(rset.next()){
		    	String name=rset.getString("name");
		    	String value=rset.getString("value");
		    	
		    	m_DbConfiguration.put(name,value);
		    }
		    rset.close();
		    stmt.close();
		    
		    m_DatabaseAdapter=this;
		}
		catch(Exception e){
			e.printStackTrace();
		}
		
	}
	
	public String getConfiguration(String name){
		Object o=m_DbConfiguration.get(name);
		if(o!=null){
			return (String)o;
		}
		return null;
	}
	
	public void postCommand(IDbCommand command) throws Exception {
		if(command.getImplVersion()==0x0100){ // Version must be 1.0
			try{
				command.execCommand(m_SqlConnection,m_DbConfiguration);
			}catch(java.sql.SQLException e){
				// Try Re-Connection
				initConnection();
				command.execCommand(m_SqlConnection, m_DbConfiguration);
			}
		}else{
			throw new Exception("Unsupported Version Command");
		}
	}
}
