package org.livee.data.db;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.exception.LiveEException;
import org.livee.data.exception.SensorNotFoundLiveEException;
import org.livee.data.expr.AbstractData;
import org.livee.data.expr.DataFactory;


/**
 * @author Hideya Ochiai, The University of Tokyo
 * 
 */
public class GetArchiveCombinedDataCommand implements IDbCommand {

	public class ValueElement{
		public String id;
		public String time;
		public AbstractData value;
	}
	
	private String m_ID;
	private String m_StartTimestamp;
	private String m_EndTimestamp;
	
	private java.util.TimeZone m_TimeZone;
	private String m_Locale;
	
	private boolean m_Succeeded;
	private AbstractData m_CombinedSensor;
	
	public GetArchiveCombinedDataCommand(String id, String start, String end, java.util.TimeZone tz,String locale){
	
		m_ID=id;
		
		m_StartTimestamp=start;
		m_EndTimestamp=end;
		m_TimeZone=tz;
		m_Locale=locale;
				
		m_Succeeded=false;
	}
	
	public boolean succeeded(){
		return m_Succeeded;
	}
	
	public AbstractData getCombinedSensor(){
		return m_CombinedSensor;
	}
	
	public void execCommand(Connection connection, Map conf) throws Exception {
	
		DataFactory df=DataFactory.getInstance();
		m_CombinedSensor=null;
		java.sql.Statement stmt=null;

		
		try{
			String sql;
			stmt=connection.createStatement();
			
			java.sql.ResultSet rset;
						
			// Combined Archive Reader
			sql="SELECT id FROM sensor.combined WHERE id=\'"+m_ID+"\'";
			rset=stmt.executeQuery(sql);
			if(rset.next()){
				m_CombinedSensor=df.create("combined");
				m_CombinedSensor.setAttribute("id", rset.getString("id"));
			}else{
				// Not Found
				m_CombinedSensor=null;
				throw new SensorNotFoundLiveEException(m_ID);
			}
			rset.close();
	
			// Element Archive Reader
			java.util.Map m_ElementMap=new java.util.Hashtable();
			sql="SELECT id FROM sensor.element WHERE working=\'t\' AND combined_id=\'"+m_ID+"\'";
			rset=stmt.executeQuery(sql);
			while(rset.next()){
				AbstractData element=df.create("element");
				m_CombinedSensor.append(element);
				
				// Set "element"
				String id=rset.getString("id");
				element.setAttribute("id", id);
				m_ElementMap.put(id, element);
			}
			rset.close();
						
			// Data Archive Reader
			if(m_EndTimestamp==null){
				sql="SELECT " +
						"raw.archive.id, " +
						"raw.archive.time, " +
						"raw.archive.attrs, " +
						"raw.archive.value, " +
						"raw.archive.sValue " +
					"FROM sensor.element RIGHT JOIN raw.archive " +
					"ON sensor.element.id=raw.archive.id " +
					"WHERE sensor.element.combined_id=\'"+m_ID+"\' AND raw.archive.time>=\'"+m_StartTimestamp+"\' ORDER BY time";
			}else{
				sql="SELECT " +
						"raw.archive.id, " +
						"raw.archive.time, " +
						"raw.archive.attrs, " +
						"raw.archive.value, " +
						"raw.archive.sValue " +
					"FROM sensor.element RIGHT JOIN raw.archive " +
					"ON sensor.element.id=raw.archive.id " +
					"WHERE sensor.element.combined_id=\'"+m_ID+"\' AND raw.archive.time>=\'"+m_StartTimestamp+"\' AND raw.archive.time<=\'"+m_EndTimestamp+"\' ORDER BY time";
			}
			rset=stmt.executeQuery(sql);
			while(rset.next()){
				AbstractData value=df.create("value");
				String id=rset.getString("id");
				
				// Set Attributes of "value"
				String time=null;
				try{
					time=org.livee.util.W3CTimestamp.toString(rset.getTimestamp("time"),m_TimeZone);
				}
				catch(Exception e){
					time=rset.getString("time");
				}
				if(time!=null){
					value.setAttribute("time",time);
				}

				String attrs=rset.getString("attrs");
				try{
					org.json.JSONObject jobj=new org.json.JSONObject(attrs);
					java.util.Iterator itr=jobj.keys();
					while(itr.hasNext()){
						String key=(String)itr.next();
						value.setAttribute(key, jobj.getString(key));
					}
				}
				catch(Exception exp){
					exp.printStackTrace();
				}
				
				String text=rset.getString("value");
				if(text==null || text.equals("")){
					text=rset.getString("sValue");
				}
				if(text!=null){
					value.setTextContent(text);
				}
				
				Object obj=m_ElementMap.get(id);
				if(obj!=null){
					AbstractData element=(AbstractData)obj;
					element.append(value);
				}
			}
			rset.close();
			m_Succeeded=true;
		}
		catch(Exception e){
			throw e;
		}
		finally{
			stmt.close();
		}

	}

	public String getCommandName() {
		return "GetArchiveCombinedDataArchive";
	}

	public short getImplVersion() {
		return 0x0100;
	}
}
