package org.livee.data.db;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.exception.SensorNotFoundLiveEException;
import org.livee.data.expr.AbstractData;


/**
 * GetCombinedProfileCommand is obsolete
 * @author Hideya Ochiai, Akihiro Sugiyama, The University of Tokyo
 * @see GetProfileCommand
 */
public class GetCombinedProfileCommand implements IDbCommand {


	/**
	 * Request
	 */
	String m_ID;

	/**
	 * Response
	 */

	AbstractData m_CombinedSensor = null;

	private GetCombinedProfileCommand() {
	}

	protected GetCombinedProfileCommand(String id) {
		m_ID = id;
		m_Succeeded = false;
	}

	boolean m_Succeeded;

	public boolean succeeded() {
		return m_Succeeded;
	}

	public AbstractData getResult() {
		return m_CombinedSensor;
	}

	public String getCommandName() {
		return "GetCombinedProfile";
	}

	public short getImplVersion() {
		return 0x0100;
	}

	public void execCommand(Connection connection, Map conf) throws Exception {
		
		java.sql.Statement stmt=null;

		try {
			stmt = connection.createStatement();
			String sql = "SELECT created, latitude,longitude,attrs FROM profile.combined WHERE id='"
					+ m_ID + "';";
			java.sql.ResultSet rset = stmt.executeQuery(sql);

			org.livee.data.expr.DataFactory df = org.livee.data.expr.DataFactory
					.getInstance();
			m_CombinedSensor = df.create("combined");
			m_CombinedSensor.setAttribute("id", m_ID);

			if (rset.next()) {
				m_CombinedSensor.setAttribute("created", rset
						.getString("created"));
				m_CombinedSensor.setAttribute("latitude", rset
						.getString("latitude"));
				m_CombinedSensor.setAttribute("longitude", rset
						.getString("longitude"));

				try {
					org.json.JSONObject jo = new org.json.JSONObject(rset
							.getString("attrs"));
					java.util.Iterator itr = jo.keys();
					while (itr.hasNext()) {
						String key = (String) itr.next();
						m_CombinedSensor.setAttribute(key, jo.getString(key));
					}
				} catch (Exception e) {
					e.printStackTrace();
				}

			} else {
				// Error not found..
				m_CombinedSensor=null;
				throw new SensorNotFoundLiveEException(m_ID);
			}
			rset.close();

			m_Succeeded = true;
		} catch (Exception e) {
			throw e;
		}
		finally{
			stmt.close();
		}
	}


}
