package org.livee.data.db;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.exception.LiveEException;
import org.livee.data.expr.AbstractData;
import org.livee.data.expr.DataFactory;

import org.livee.util.W3CTimestamp;

/**
 * @author Hideya Ochiai, The University of Tokyo
 * 
 */
public class GetViewProfileArchiveCommand implements IDbCommand {

	private String m_ID;
	private java.util.Calendar m_StartTimestamp;
	private java.util.Calendar m_EndTimestamp;
	
	private java.util.TimeZone m_TimeZone;
	private String m_Locale;
	
	private boolean m_Succeeded;
	private AbstractData m_Collection;
	
	public GetViewProfileArchiveCommand(String id,java.util.Calendar start, java.util.Calendar end, java.util.TimeZone tz,String locale){
	
		m_ID=id;
		
		m_StartTimestamp=start;
		m_EndTimestamp=end;
		m_TimeZone=tz;
		m_Locale=locale;
				
		m_Succeeded=false;
	}
	
	public boolean succeeded(){
		return m_Succeeded;
	}
	
	public AbstractData getCollection(){
		return m_Collection;
	}
	
	public void execCommand(Connection connection, Map conf) throws Exception {
	
		DataFactory df=DataFactory.getInstance();
		m_Collection=df.create("collection");
		String language_code=null;
		String runtime_language_code=null;
		java.sql.Statement stmt=null;
		
		try{
			String sql;
			stmt=connection.createStatement();
			
			//if(m_TimeZone!=null){
			//	sql="SET TIMEZONE TO "+(m_TimeZone.getRawOffset()/3600000);
			//	stmt.execute(sql);
			//}
			
			sql="SELECT language_code FROM admin.locale WHERE name=\'"+m_Locale+"\'";
			java.sql.ResultSet rset=stmt.executeQuery(sql);
			if(rset.next()){
				language_code=rset.getString(1);
				runtime_language_code="_"+language_code;
			}else{
				language_code="eng";
				runtime_language_code="_eng";
			}
			
			// Combined Archive Reader
			java.util.Collection m_CombinedCollection=new java.util.ArrayList();
			if(m_EndTimestamp==null){
				sql="SELECT id, created, expired, latitude, longitude, attrs FROM profile.combined_archive WHERE id=\'"+m_ID+"\' AND (expired>\'"+m_StartTimestamp.getTime()+"\' OR expired IS NULL) ORDER BY created";
			}else{
				sql="SELECT id, created, expired, latitude, longitude, attrs FROM profile.combined_archive WHERE id=\'"+m_ID+"\' AND (expired>\'"+m_StartTimestamp.getTime()+"\' OR expired IS NULL) AND created<\'"+m_EndTimestamp.getTime()+"\' ORDER BY created";
			}
			rset=stmt.executeQuery(sql);
			while(rset.next()){
				AbstractData combined=df.create("combined");
				m_Collection.append(combined);
				
				String id=rset.getString("id");
				combined.setAttribute("id", id);
				combined.setAttribute("created", W3CTimestamp.toString(rset.getTimestamp("created"),m_TimeZone));
				String expired=W3CTimestamp.toString(rset.getTimestamp("expired"),m_TimeZone);
				if(expired!=null){
					combined.setAttribute("expired", expired);
				}
				combined.setAttribute("latitude", rset.getString("latitude"));
				combined.setAttribute("longitude", rset.getString("longitude"));
				String attrs=rset.getString("attrs");
				try{
					java.util.Map map=new java.util.Hashtable();
					org.json.JSONObject jobj=new org.json.JSONObject(attrs);
					java.util.Iterator itr=jobj.keys();
					while(itr.hasNext()){
						String key=(String)itr.next();
						
						int index=key.lastIndexOf("_");
						if(language_code!=null && index>0){
							String generic_key=key.substring(0,index);
							if(key.endsWith(runtime_language_code)){
								map.put(generic_key, key);
							}else if(key.endsWith("_eng")){
								if(!map.containsKey(generic_key)){
									map.put(generic_key, key);
								}
							}
						}else if(!map.containsKey(key)){
							map.put(key, key);
						}
					}

					itr=map.keySet().iterator();
					while(itr.hasNext()){
						String generic_key=(String)itr.next();
						String key=(String)map.get(generic_key);
						combined.setAttribute(generic_key, jobj.getString(key));
					}

				}
				catch(Exception e){
					e.printStackTrace();
				}
				m_CombinedCollection.add(combined);
			}
			rset.close();
	
			// Element Archive Reader
			java.util.Collection m_ElementCollection=new java.util.ArrayList();
			if(m_EndTimestamp==null){
				sql="SELECT " +
						"sensor.element.id, " +
						"sensor.element.combined_id, " +
						"profile.element_archive.created, " +
						"profile.element_archive.expired, " +
						"profile.element_archive.sensorType, " +
						"profile.element_archive.attrs " +
					"FROM sensor.element,profile.element_archive " +
					"WHERE sensor.element.id=profile.element_archive.id " +
					"AND sensor.element.combined_id=\'"+m_ID+"\' AND (expired>\'"+m_StartTimestamp.getTime()+"\' OR expired IS NULL) ORDER BY created";
			}else{
				sql="SELECT " +
						"sensor.element.id, " +
						"sensor.element.combined_id, " +
						"profile.element_archive.created, " +
						"profile.element_archive.expired, " +
						"profile.element_archive.sensorType, " +
						"profile.element_archive.attrs " +
					"FROM sensor.element,profile.element_archive " +
					"WHERE sensor.element.id=profile.element_archive.id " +
					"AND sensor.element.combined_id=\'"+m_ID+"\' AND (expired>\'"+m_StartTimestamp.getTime()+"\' OR expired IS NULL) AND created<\'"+m_EndTimestamp.getTime()+"\' ORDER BY created";
			}
			rset=stmt.executeQuery(sql);
			while(rset.next()){
				AbstractData element=df.create("element");
								
				// Set Attributes of "element"
				element.setAttribute("id", rset.getString("id"));
				element.setAttribute("created", W3CTimestamp.toString(rset.getTimestamp("created"),m_TimeZone));
				String expired=W3CTimestamp.toString(rset.getTimestamp("expired"),m_TimeZone);
				if(expired!=null){
					element.setAttribute("expired", expired);
				}
				element.setAttribute("sensorType",rset.getString("sensorType"));
				
				String attrs=rset.getString("attrs");
				try{
					org.json.JSONObject jobj=new org.json.JSONObject(attrs);
					java.util.Iterator itr=jobj.keys();
					while(itr.hasNext()){
						String key=(String)itr.next();
						element.setAttribute(key, jobj.getString(key));
					}
				}
				catch(Exception e){
				//	e.printStackTrace();
				}
				m_ElementCollection.add(element);
			}
			rset.close();
			
			
			// Combined and Element Binder (Fully bind O(n*n))
			java.util.Iterator citr=m_CombinedCollection.iterator();
			while(citr.hasNext()){
				AbstractData combined=(AbstractData)citr.next();
				String created=combined.getAttribute("created");
				String expired=combined.getAttribute("expired");
				
				java.util.Iterator eitr=m_ElementCollection.iterator();
				while(eitr.hasNext()){
					AbstractData element=(AbstractData)eitr.next();
					String c=element.getAttribute("created");
					String e=element.getAttribute("expired");
					if(expired==null){
						if(e==null || e.compareTo(created)>0){
							combined.append(element);
						}
					}else{
						if(c.compareTo(expired)<0 && (e==null || e.compareTo(created)>0)){
							combined.append(element);
						}
					}
					
				}
			}
			
			m_Succeeded=true;
		}
		catch(Exception e){
			throw e;
		}
		finally{
			stmt.close();
		}

	}

	public String getCommandName() {
		return "GetViewProfileArchive";
	}

	public short getImplVersion() {
		return 0x0100;
	}

}
