package org.livee.data.db;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.exception.LiveEException;
import org.livee.data.exception.SensorNotFoundLiveEException;
import org.livee.data.expr.AbstractData;
import org.livee.data.expr.DataFactory;

/**
 * @author Hideya Ochiai, The University of Tokyo
 * 
 */
public class GetViewProfileCommand implements IDbCommand {

	private String m_ID;
	private java.util.TimeZone m_TimeZone;
	private String m_Locale;
	
	private boolean m_Succeeded;
	private AbstractData m_CombinedSensor;
	
	public GetViewProfileCommand(String id,java.util.TimeZone tz,String locale){
		m_ID=id;
		m_TimeZone=tz;
		m_Locale=locale;
				
		m_Succeeded=false;
	}
	
	public boolean succeeded(){
		return m_Succeeded;
	}
	
	public AbstractData getCombinedSensor(){
		return m_CombinedSensor;
	}
	
	public void execCommand(Connection connection, Map conf) throws Exception {
	
		DataFactory df=DataFactory.getInstance();
		m_CombinedSensor=df.create("combined");
		String language_code=null;
		String runtime_language_code=null;
		java.sql.Statement stmt=null;
		
		try{
			String sql;
			stmt=connection.createStatement();
			
			
			sql="SELECT language_code FROM admin.locale WHERE name=\'"+m_Locale+"\'";
			java.sql.ResultSet rset=stmt.executeQuery(sql);
			if(rset.next()){
				language_code=rset.getString(1);
				runtime_language_code="_"+language_code;
			}else{
				language_code="eng";
				runtime_language_code="_eng";
			}
						
			sql="SELECT sensor.combined.id, created, latitude, longitude, attrs FROM sensor.combined LEFT JOIN profile.combined ON sensor.combined.id=profile.combined.id WHERE sensor.combined.working=\'t\' AND sensor.combined.id=\'"+m_ID+"\'";
			rset=stmt.executeQuery(sql);
			if(rset.next()){
				String id=rset.getString("id");
				m_CombinedSensor.setAttribute("id", id);
				m_CombinedSensor.setAttribute("latitude", rset.getString("latitude"));
				m_CombinedSensor.setAttribute("longitude", rset.getString("longitude"));
				String attrs=rset.getString("attrs");
				try{
					java.util.Map map=new java.util.Hashtable();
					org.json.JSONObject jobj=new org.json.JSONObject(attrs);
					java.util.Iterator itr=jobj.keys();
					while(itr.hasNext()){
						String key=(String)itr.next();
						
						int index=key.lastIndexOf("_");
						if(language_code!=null && index>0){
							String generic_key=key.substring(0,index);
							if(key.endsWith(runtime_language_code)){
								map.put(generic_key, key);
							}else if(key.endsWith("_eng")){
								if(!map.containsKey(generic_key)){
									map.put(generic_key, key);
								}
							}
						}else if(!map.containsKey(key)){
							map.put(key, key);
						}
					}

					itr=map.keySet().iterator();
					while(itr.hasNext()){
						String generic_key=(String)itr.next();
						String key=(String)map.get(generic_key);
						m_CombinedSensor.setAttribute(generic_key, jobj.getString(key));
					}

				}
				catch(Exception e){
				//	e.printStackTrace();
				}
			}else{
				// Not found;
				m_CombinedSensor=null;
				throw new SensorNotFoundLiveEException(m_ID);
			}
			rset.close();
			
			sql="SELECT " +
					"profile.element.id, " +
					"profile.element.sensorType, " +
					"profile.element.attrs " +
				"FROM sensor.element RIGHT JOIN profile.element " +
				"ON sensor.element.id=profile.element.id " +
				"WHERE sensor.element.combined_id=\'"+m_ID+"\';";
			
			rset=stmt.executeQuery(sql);
			while(rset.next()){
				AbstractData element=df.create("element");
				m_CombinedSensor.append(element);
				
				// Set Attributes of "element"
				element.setAttribute("id", rset.getString("id"));
				element.setAttribute("sensorType",rset.getString("sensorType"));
				
				String attrs=rset.getString("attrs");
				try{
					org.json.JSONObject jobj=new org.json.JSONObject(attrs);
					java.util.Iterator itr=jobj.keys();
					while(itr.hasNext()){
						String key=(String)itr.next();
						element.setAttribute(key, jobj.getString(key));
					}
				}
				catch(Exception e){
				//	e.printStackTrace();
				}
				
			}
			rset.close();
			
			m_Succeeded=true;
		}
		catch(Exception e){
			throw e;
		}
		finally{
			stmt.close();
		}

	}

	public String getCommandName() {
		return "GetViewProfile";
	}

	public short getImplVersion() {
		return 0x0100;
	}

}
