package org.livee.data.db;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/**
 * IDbCommand Interface for DatabaseAdapter
 * Deal the implemented class of this interface as an command by posting the 
 * instance into "postCommand" method of the DatabaseAdapter.
 * DatabaseAdapter then calls execCommand of this interface with the instance of "java.sql.Connection"
 * which is connected to the database. 
 * @author Hideya Ochiai, The University of Tokyo
 * @version 1.0
 */
public interface IDbCommand {
	/**
	 * @return The version of the implementation process
	 * e.g., 0x0102 <==> Version 1.2 
	 */
	public short getImplVersion();
	
	/**
	 * @return The command name of the implementation process
	 */
	public String getCommandName();
	
	/**
	 * This method must process a command with the provided "connection" 
	 * @param connection Database Connection
	 * @throws Exception
	 */
	public void execCommand(java.sql.Connection connection,java.util.Map conf) throws Exception;
}
