package org.livee.data.db;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Iterator;
import java.util.Map;

import org.json.JSONObject;
import org.livee.data.expr.AbstractData;

/**
 * @author Hideya Ochiai, The University of Tokyo
 * 
 */
public class RestoreAdminCommand implements IDbCommand {

	/**
	 * Request
	 */
	AbstractData m_AdminReplicate = null;

	/**
	 * Response
	 */
	boolean m_Succeeded;

	private RestoreAdminCommand() {
	}

	protected RestoreAdminCommand(AbstractData adminReplicate) {
		m_AdminReplicate = adminReplicate;
		m_Succeeded = false;
	}

	public String getCommandName() {
		return "RestoreAdminCommand";
	}

	public short getImplVersion() {
		return 0x0100;
	}

	public boolean succeeded() {
		return m_Succeeded;
	}

	public void execCommand(Connection connection, Map conf) throws Exception {

		java.util.Iterator itr=m_AdminReplicate.getChildren().iterator();
		while(itr.hasNext()){
			AbstractData data=(AbstractData)itr.next();
			String type=data.getDataType();
			IDbCommand command=null;
			if(type.equals("AdminConfig")){
				command=new SetAdminConfigCommand(data);
			// Moved to AdminConfig
			//}else if(type.equals("AdminNeighbor")){
			//	command=new SetAdminNeighborCommand(data);
			}else if(type.equals("LocaleCollection")){
				command=new SetAdminLocaleSchemaCommand(data);
			}else if(type.equals("ProfileSchema")){
				command=new SetAdminProfileSchemaCommand(data);
			}else if(type.equals("SensorCollection")){
				command=new SetAdminProfileReplicateCommand(data);
			}
			if(command!=null){
				command.execCommand(connection, conf);
			}
		}
		
		m_Succeeded=true;
	}

}
