package org.livee.data.db;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import java.util.*;

import org.livee.data.expr.AbstractData;
import org.json.*;

/**
 * SetElementProfileCommand is obsolete
 * @author Hideya Ochiai, Akihiro Sugiyama, The University of Tokyo
 * @see SetProfileCommand
 */
public class SetElementProfileCommand implements IDbCommand {

	/**
	 * Request
	 */
	AbstractData m_ElementSensor = null;

	/**
	 * Response
	 */
	boolean m_Succeeded;

	private SetElementProfileCommand() {
	}

	protected SetElementProfileCommand(AbstractData ElementSensor) {
		m_ElementSensor = ElementSensor;
		m_Succeeded = false;
	}

	public String getCommandName() {
		return "SetElementProfileCommand";
	}

	public short getImplVersion() {
		return 0x0100;
	}

	public boolean succeeded() {
		return m_Succeeded;
	}

	public void execCommand(Connection connection, Map conf) throws Exception {
		java.sql.Statement stmt = null;
		
		try {
			stmt = connection.createStatement();
			String sql;

			String id = null;
			String created = null;
			String sensorType = null;
			JSONObject attrs = new JSONObject(); // Attribute Iteration Map
			// map2 =
			Map map = m_ElementSensor.getAttributes();
			Iterator itr = map.keySet().iterator();
			while (itr.hasNext()) {
				String key = (String) itr.next();
				String value = (String) map.get(key);

				if (key.equals("id")) {
					id = value;
				} else if (key.equals("created")) {
					created = value;
				} else if (key.equals("sensorType")) {
					sensorType = value;
				} else {
					attrs.put(key, value);
				}
			}

			if (created == null){
				created = java.util.Calendar.getInstance().getTime().toString();
			}
			
			sql = "UPDATE profile.element_archive SET expired=\'" + created
					+ "\' WHERE id=\'" + id + "\' and expired IS NULL;";
			//System.out.println(sql);

			stmt.execute("START TRANSACTION;");
			stmt.executeUpdate(sql);

			sql = "INSERT INTO profile.element_archive (id, created, expired, sensorType, attrs) values (\'"
					+ id
					+ "\', \'"
					+ created
					+ "\', null, \'"
					+ sensorType
					+ "\', \'" + attrs + "\');";


			stmt.execute(sql);
			stmt.execute("COMMIT;");

		} catch (Exception e) {
			stmt.execute("ROLLBACK");
			throw e;
		}
		finally{
			stmt.close();
		}
	}

}
