package org.livee.data.db;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.expr.AbstractData;
import org.livee.data.expr.DataFactory;


/**
 * SetProfileSchemaCommand is obsolete
 * @author Hideya Ochiai, The University of Tokyo
 * @see SetAdminProfileSchemaCommand
 */
public class SetProfileSchemaCommand implements IDbCommand {
	
	private boolean m_Succeeded;
	private AbstractData m_ProfileSchema;
	
	protected SetProfileSchemaCommand(AbstractData profileSchema){
		m_ProfileSchema=profileSchema;
		m_Succeeded=false;
	}
	
	public boolean succeeded(){
		return m_Succeeded;
	}

	public void execCommand(Connection connection, Map conf) throws Exception {
		
		String sql;
		java.sql.Statement stmt=null;

		try{

			stmt = connection.createStatement();
			stmt.execute("START TRANSACTION;");
			stmt.execute("DELETE FROM profile.schema;");

			java.util.Iterator itr=m_ProfileSchema.getChildren().iterator();
			while(itr.hasNext()){
				AbstractData schema=(AbstractData)itr.next();
				String name=schema.getAttribute("name");
				String type=schema.getAttribute("type");
				String value=schema.getAttribute("value");
				String description=schema.getAttribute("description");
				String visibility=schema.getAttribute("visibility");
				String multilanguage=schema.getAttribute("multilanguage");
				String clazz=schema.getAttribute("class");
				
				if(name==null || type==null){
					continue;
				}
				StringBuilder sb=new StringBuilder("INSERT INTO profile.schema (name,type,class,value,multilanguage,description,visibility) VALUES (\'");
				
				sb.append(name);
				sb.append("\',\'");
				sb.append(type);
				sb.append("\',");
				
				if(clazz!=null){
					sb.append("\'");
					sb.append(clazz.replaceAll("\'","\\\\\'"));
					sb.append("\',");
				}else{
					sb.append("\'.*\',");
				}

				if(value!=null){
					sb.append("\'");
					sb.append(value.replaceAll("\'","\\\\\'"));
					sb.append("\',");

					// Regular Expression Check
					java.util.regex.Pattern.compile(value);
				}else{
					sb.append("\'.*\',");
				}
				
				if(multilanguage!=null){
					sb.append("'t',");
				}else{
					sb.append("null,");
				}
				
				if(description!=null){
					sb.append("\'");
					sb.append(description.replaceAll("\'","\\\\\'"));
					sb.append("\',");
				}else{
					sb.append("null,");
				}
				
				if(visibility!=null){
					sb.append("\'");
					sb.append(visibility.replaceAll("\'","\\\\\'"));
					sb.append("\');");
				}else{
					sb.append("null);");
				}
				sql=sb.toString();

				// System.out.println(sql);
				
				stmt.execute(sql);
			}
			
			
			stmt.execute("COMMIT;");
			m_Succeeded=true;
		}
		catch(Exception e){
			stmt.execute("ROLLBACK;");
			throw e;
		}
		finally{
			stmt.close();
		}
	}

	public String getCommandName() {
		return "SetProfileSchemaCommand";
	}

	public short getImplVersion() {
		return 0x0100;
	}

}
