package org.livee.data.db;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.json.JSONObject;
import org.livee.data.exception.AuthenticationLiveEException;
import org.livee.data.expr.AbstractData;

/**
 * @author Hideya Ochiai, The University of Tokyo
 * 
 */
public class UploadCombinedCommand implements IDbCommand {

	/**
	 * Request
	 */
	AbstractData m_CombinedSensor = null;
	String m_ID=null;
	String m_Passwd = null;

	/**
	 * Response
	 */
	boolean m_Succeeded;

	private UploadCombinedCommand() {
	}

	protected UploadCombinedCommand(AbstractData element, String passwd) {
		m_CombinedSensor = element;
		try {
			m_ID=element.getAttribute("id");
			m_Passwd = org.livee.util.Security.SHA1(passwd);
		} catch (Exception e) {
			e.printStackTrace();
		}
		m_Succeeded = false;
	}

	public String getCommandName() {
		return "UploadCombinedCommand";
	}

	public short getImplVersion() {
		return 0x0100;
	}

	public boolean succeeded() {
		return m_Succeeded;
	}

	public void execCommand(Connection connection, Map conf) throws Exception {

		java.sql.Statement stmt = null;
		
		// If m_ID is working (check at sensor.element) 
		// UPDATE (INSERT INTO) raw.latest, INSERT into raw.daily, raw.archive
		
		try{
		
			stmt = connection.createStatement();
			
			String sql = "SELECT id FROM sensor.combined WHERE id=\'"+m_ID+"\' AND passwd=\'"+m_Passwd+"\';";
		
			java.sql.ResultSet rset=stmt.executeQuery(sql);
			if(!rset.next()){
				// The m_ID is not registered or authentication error.
				throw new AuthenticationLiveEException(m_ID,m_Passwd);
			}
			rset.close();
		
			java.util.Iterator itr0=m_CombinedSensor.getChildren().iterator();
			while(itr0.hasNext()){
				AbstractData element=(AbstractData)itr0.next();

				String id=element.getAttribute("id");
				String latestTime=null;
				String latestAttrs=null;
				String latestValue=null;
				java.util.Iterator itr=element.getChildren().iterator();
				while(itr.hasNext()){
					AbstractData value=(AbstractData)itr.next();
					JSONObject attrs = new JSONObject();
					
					java.util.Map attrMap=value.getAttributes();
					try{
						latestTime=org.livee.util.W3CTimestamp.parseToDateString((String)attrMap.get("time"));
						attrMap.remove("time");
					
						java.util.Iterator attr_itr=attrMap.keySet().iterator();
						while(attr_itr.hasNext()){
							String k=(String)attr_itr.next();
							String v=(String)attrMap.get(k);
							attrs.put(k, v);
						}
						latestAttrs=attrs.toString();
						latestValue=value.getTextContent();
						
						insertIntoLogDb(stmt,id,latestTime,latestAttrs,latestValue);
						
					}catch(Exception e){
						
					}
				}
				updateLatestDb(stmt,id,latestTime,latestAttrs,latestValue);
			}
		
			m_Succeeded=true;
		}
		catch(Exception e){
			throw e;
		}
		finally{
			stmt.close();
		}

	}
	
	private boolean updateLatestDb(java.sql.Statement stmt,String id,String time,String attrs,String value) throws Exception {
		
		if(id==null || time==null || attrs==null || value==null){
			return false;
		}
		
		String sql;
		String insert_sql;
		try{
			Double.parseDouble(value);
			sql="UPDATE raw.latest SET time=\'"+time+"\', attrs=\'"+attrs.replaceAll("\'", "\\\\\'")+"\', value="+value+", sValue=null WHERE id=\'"+id+"\'";
			insert_sql="(id, time, attrs, value, sValue) VALUES (\'"+id+"\',\'"+time+"\',\'"+attrs.replaceAll("\'", "\\\\\'")+"\',"+value+",null);";
		}
		catch(Exception e){
			sql="UPDATE raw.latest SET time=\'"+time+"\', attrs=\'"+attrs.replaceAll("\'", "\\\\\'")+"\', value=null, sValue=\'"+value+"\' WHERE id=\'"+id+"\'";
			insert_sql="(id, time, attrs, value, sValue) VALUES (\'"+id+"\',\'"+time+"\',\'"+attrs.replaceAll("\'", "\\\\\'")+"\',null,\'"+value+"\');";
		}
		
		if (stmt.executeUpdate(sql) == 0){
			// "Raw" did not exist ==> INSERT INTO raw.latest
			sql="INSERT INTO raw.latest "+insert_sql;
			stmt.execute(sql);
		}
		
		return true;
	}
	
	private boolean insertIntoLogDb(java.sql.Statement stmt, String id,String time,String attrs,String value) throws Exception {

		if(id==null || time==null || attrs==null || value==null){
			return false;
		}

		String sql;
		String insert_sql;
		try{
			Double.parseDouble(value);
			insert_sql="(id, time, attrs, value, sValue) VALUES (\'"+id+"\',\'"+time+"\',\'"+attrs.replaceAll("\'", "\\\\\'")+"\',"+value+",null);";
		}
		catch(Exception e){
			insert_sql="(id, time, attrs, value, sValue) VALUES (\'"+id+"\',\'"+time+"\',\'"+attrs.replaceAll("\'", "\\\\\'")+"\',null,\'"+value+"\');";
		}
		
		//sql="INSERT INTO raw.daily "+insert_sql;
		//stmt.execute(sql);
		
		sql="INSERT INTO aggregation.work "+insert_sql;
		stmt.execute(sql);
		
		sql="INSERT INTO raw.archive "+insert_sql;
		stmt.execute(sql);
		
		return true;
	}


}
