package org.livee.data.expr;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.util.Collection;

/**
 * Data Handler of Data Object Model This class also provides serialization method for archived expression.
 * @author  Hideya Ochiai, The University of Tokyo
 * @version  1.0
 */
public interface AbstractData {

	/**
	 * Append the child into this Data
	 * The implementation can identify the type of child by using getDataType method.
	 * @param child
	 */
	public void append(AbstractData child) throws org.livee.data.exception.LiveEException;
	
	/**
	 * @return child abstract data entities
	 */
	public Collection getChildren();
	
	/**
	 * @param child
	 */
	public void remove(AbstractData child);
	
	
	/**
	 * @param name
	 * @param value
	 */
	public void setAttribute(String name,String value);
	
	/**
	 * @param name
	 * @return
	 */
	public String getAttribute(String name);
	
	/**
	 * @return AttributeMap
	 */
	public java.util.Map getAttributes();
	
	/**
	 * @param name
	 */
	public void removeAttribute(String name);
	
	/**
	 * @param  text
	 * @uml.property  name="textContent"
	 */
	public void setTextContent(String text);
	
	/**
	 * @return
	 * @uml.property  name="textContent"
	 */
	public String getTextContent();
	
	/**
	 * @return Data Type
	 */
	public String getDataType();
	
	/**
	 * Serialize into (or generate) a XML expression
	 * @return
	 */
	public org.w3c.dom.Element getXml(org.w3c.dom.Document doc);
}
