package org.livee.data.expr;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.util.Collection;
import java.util.Map;

import org.livee.data.exception.OutOfMaxSizeLiveEException;
import org.livee.data.tools.LogManager;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.livee.data.exception.LiveEException;

/**
 * 
 * @author Hideya Ochiai, The University of Tokyo
 */
public class ElementSensorData implements AbstractData {

	// Size Restriction System
	private static boolean m_bSizeBorderLoaded=false;
	private static int m_WarningBorder;
	private static int m_ErrorBorder;
	private static void loadSizeBorder(){
		org.livee.data.tools.ConfigManager config=org.livee.data.tools.ConfigManager.getInstance();
		int warning=config.getIntParameter("/c:liveeConfiguration/c:restriction/c:maxValueCollectionCount/c:warning/text()");
		int error=config.getIntParameter("/c:liveeConfiguration/c:restriction/c:maxValueCollectionCount/c:error/text()");
		if(warning>0){
			m_WarningBorder=warning;
		}else{
			m_WarningBorder=16384;  // recommended default 16384 
		}
		if(error>0){
			m_ErrorBorder=error;
		}else{
			m_ErrorBorder=32768; // recommended default 32768
		}
		m_bSizeBorderLoaded=true;
	}
	
	String m_ID;
	java.util.Map m_Attributes;
	java.util.Collection m_ValueDataCol;
	
	protected ElementSensorData(){
		m_Attributes=new java.util.Hashtable();
		m_ValueDataCol=new java.util.ArrayList();
	}
	
	public void append(AbstractData child) throws LiveEException {
		if(child==null){
			return ;
		}
		
		int size=m_ValueDataCol.size();
		if(m_bSizeBorderLoaded==false){
			loadSizeBorder();
		}
		if(size>m_WarningBorder){
			if(size>m_ErrorBorder){
				// error and exception
				String errorMessage= "The size of value exceeded error level("+m_ErrorBorder+"): "+size;
				LogManager.postError("restriction.maxValueCollectionCount.error",errorMessage);
				throw new OutOfMaxSizeLiveEException("valueCollection",m_ErrorBorder);
			}else{
				// warning
				LogManager.postWarning("restriction.maxValueCollectionCount.warning", "The size of value exceeded warning level("+m_WarningBorder+"): "+size);
			}
		}

		if(child!=null){
			String name=child.getDataType();
			if(name.equals("Value") || name.equals("AggValue")){
				m_ValueDataCol.add(child);
			}
		}
		
	}

	public String getAttribute(String name) {
		if(name==null){
			return null;
		}
		
		Object obj=m_Attributes.get(name);
		if(obj!=null){
			return (String)obj;
		}
		return null;
	}

	public Map getAttributes() {
		return m_Attributes;
	}

	public Collection getChildren() {
		return m_ValueDataCol;
	}

	public String getDataType() {
		return "ElementSensor";
	}

	public String getTextContent() {
		return null;
	}

	public void remove(AbstractData child) {
		if(child!=null){
			m_ValueDataCol.remove(child);
		}
	}

	public void removeAttribute(String name) {
		if(name!=null){
			m_Attributes.remove(name);
		}
	}

	public void setAttribute(String name, String value) {
		if(name!=null){
			if(name.equals("id")){
				m_ID=value;
			}
			if(value!=null){
				m_Attributes.put(name, value);
			}
		}
	}

	public void setTextContent(String text) {}

	public Element getXml(Document doc) {
		
		String ns="http://live-e.org/DataType/2007/03/";
		Element element=doc.createElementNS(ns, "sensor");
		
		java.util.Iterator itr=m_Attributes.keySet().iterator();
		while(itr.hasNext()){
			Object obj=itr.next();
			if(obj!=null){
				String key=(String)obj;
				String value=(String)m_Attributes.get(key);
				
				element.setAttribute(key, value);
			}
		}

		itr=m_ValueDataCol.iterator();
		while(itr.hasNext()){
			Object obj=itr.next();
			if(obj!=null){
				AbstractData value=(AbstractData)obj;
				element.appendChild(value.getXml(doc));
			}
		}
		
		return element;
	}

}
