package org.livee.data.query;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.util.Collection;
import java.util.Map;

import org.livee.data.expr.AbstractData;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * 
 * @author Hideya Ochiai, The University of Tokyo
 */
public class FilterData implements AbstractData {

	String m_TargetClass;
	String m_AttrName;
	String m_EQ;
	String m_NEQ;
	String m_LT;
	String m_GT;
	String m_LTEQ;
	String m_GTEQ;
	String m_Contains;
	
	public void append(AbstractData child) {}

	public String getAttribute(String name) {
		if(name==null){
			return null;
		}
		if(name.equals("targetClass")){
			return m_TargetClass;
		}else if(name.equals("attrName")){
			return m_AttrName;
		}else if(name.equals("eq")){
			return m_EQ;
		}else if(name.equals("neq")){
			return m_NEQ;
		}else if(name.equals("lt")){
			return m_LT;
		}else if(name.equals("gt")){
			return m_GT;
		}else if(name.equals("lteq")){
			return m_LTEQ;
		}else if(name.equals("gteq")){
			return m_GTEQ;
		}else if(name.equals("contains")){
			return m_Contains;
		}
		return null;
	}

	public Map getAttributes() {
		Map attrs=new java.util.Hashtable();

		if(m_TargetClass!=null){
			attrs.put("targetClass",m_TargetClass);
		}
		if(m_AttrName!=null){
			attrs.put("attrName", m_AttrName);
		}
		if(m_EQ!=null){
			attrs.put("eq", m_EQ);
		}
		if(m_NEQ!=null){
			attrs.put("neq", m_NEQ);
		}
		if(m_LT!=null){
			attrs.put("lt", m_LT);
		}
		if(m_GT!=null){
			attrs.put("gt", m_GT);
		}
		if(m_LTEQ!=null){
			attrs.put("lteq", m_LTEQ);
		}
		if(m_GTEQ!=null){
			attrs.put("gteq", m_GTEQ);
		}
		if(m_Contains!=null){
			attrs.put("contains", m_Contains);
		}
		
		return attrs;
	}

	public Collection getChildren() {
		return null;
	}

	public String getDataType() {
		return "Filter";
	}

	public String getTextContent() {
		return null;
	}

	public Element getXml(Document doc) {
		
		String ns="http://live-e.org/Query/2007/03/";
		Element filter=doc.createElementNS(ns, "filter");
				
		if(m_TargetClass!=null){
			filter.setAttribute("targetClass",m_TargetClass);
		}
		if(m_AttrName!=null){
			filter.setAttribute("attrName", m_AttrName);
		}
		if(m_EQ!=null){
			filter.setAttribute("eq", m_EQ);
		}
		if(m_NEQ!=null){
			filter.setAttribute("neq", m_NEQ);
		}
		if(m_LT!=null){
			filter.setAttribute("lt", m_LT);
		}
		if(m_GT!=null){
			filter.setAttribute("gt", m_GT);
		}
		if(m_LTEQ!=null){
			filter.setAttribute("lteq", m_LTEQ);
		}
		if(m_GTEQ!=null){
			filter.setAttribute("gteq", m_GTEQ);
		}
		if(m_Contains!=null){
			filter.setAttribute("contains", m_Contains);
		}
		return filter;
		
	}

	public void remove(AbstractData child) {}

	public void removeAttribute(String name) {

		if(name==null){
			return ;
		}
		
		if(name.equals("targetClass")){
			m_TargetClass=null;
		}else if(name.equals("attrName")){
			m_AttrName=null;
		}else if(name.equals("eq")){
			m_EQ=null;
		}else if(name.equals("neq")){
			m_NEQ=null;
		}else if(name.equals("lt")){
			m_LT=null;
		}else if(name.equals("gt")){
			m_GT=null;
		}else if(name.equals("lteq")){
			m_LTEQ=null;
		}else if(name.equals("gteq")){
			m_GTEQ=null;
		}else if(name.equals("contains")){
			m_Contains=null;
		}
	}

	public void setAttribute(String name, String value) {
		
		if(name==null){
			return ;
		}
		
		if(name.equals("targetClass")){
			m_TargetClass=value;
		}else if(name.equals("attrName")){
			m_AttrName=value;
		}else if(name.equals("eq")){
			m_EQ=value;
		}else if(name.equals("neq")){
			m_NEQ=value;
		}else if(name.equals("lt")){
			m_LT=value;
		}else if(name.equals("gt")){
			m_GT=value;
		}else if(name.equals("lteq")){
			m_LTEQ=value;
		}else if(name.equals("gteq")){
			m_GTEQ=value;
		}else if(name.equals("contains")){
			m_Contains=value;
		}
	}

	public void setTextContent(String text) {
	}

	public String generatePredicateElement(String targetClass){
		if((m_TargetClass==null && targetClass==null)
				|| (m_TargetClass!=null && m_TargetClass.equals(targetClass))){
			if(m_AttrName!=null){
				
				// id Filtering
				if(m_AttrName.equals("id")){
					StringBuilder sb=new StringBuilder();
					if(m_EQ!=null){
						sb.append(" id=\'");
						sb.append(m_EQ);
						sb.append("\' AND");
					}
					if(m_NEQ!=null){
						sb.append(" id!=\'");
						sb.append(m_NEQ);
						sb.append("\' AND");
					}
					if(m_LT!=null){
						sb.append(" id<\'");
						sb.append(m_LT);
						sb.append("\' AND");
					}
					if(m_GT!=null){
						sb.append(" id>\'");
						sb.append(m_GT);
						sb.append("\' AND");
					}
					if(m_LTEQ!=null){
						sb.append(" id<=\'");
						sb.append(m_LTEQ);
						sb.append("\' AND");
					}
					if(m_GTEQ!=null){
						sb.append(" id>=\'");
						sb.append(m_LTEQ);
						sb.append("\' AND");
					}
					if(m_Contains!=null){
						sb.append(" id LIKE \'%");
						sb.append(m_Contains);
						sb.append("%\' AND");
					}
					int length=sb.length();
					sb.delete(length-4, length);
					return sb.toString();
				}
				
				// General Filtering
				StringBuilder sb=new StringBuilder(" id IN (SELECT id FROM admin.combined_index WHERE name=\'");
				sb.append(m_AttrName);
				sb.append("\' AND");
				
				org.livee.data.tools.ProfileSchemaProcessor schema=org.livee.data.tools.ProfileSchemaProcessor.getInstance();
				int type=schema.getDataType(m_AttrName);
				switch(type){
				case org.livee.data.tools.ProfileSchemaProcessor.Schema.SCHEMA_TYPE_BOOLEAN:
					if(m_EQ!=null){
						if(m_EQ.startsWith("t")){
							sb.append(" sValue=\'true\' AND");
						}else{
							sb.append(" sValue=\'false\' AND");
						}
					}
					if(m_NEQ!=null){
						if(m_EQ.startsWith("t")){
							sb.append(" sValue!=\'true\' AND");
						}else{
							sb.append(" sValue!=\'false\' AND");
						}
					}
					break;
					
				case org.livee.data.tools.ProfileSchemaProcessor.Schema.SCHEMA_TYPE_INTEGER:
					if(m_EQ!=null){
						sb.append(" iValue=");
						sb.append(m_EQ);
						sb.append(" AND");
					}
					if(m_NEQ!=null){
						sb.append(" iValue!=");
						sb.append(m_NEQ);
						sb.append(" AND");
					}
					if(m_LT!=null){
						sb.append(" iValue<");
						sb.append(m_LT);
						sb.append(" AND");
					}
					if(m_GT!=null){
						sb.append(" iValue>");
						sb.append(m_GT);
						sb.append(" AND");
					}
					if(m_LTEQ!=null){
						sb.append(" iValue<=");
						sb.append(m_LTEQ);
						sb.append(" AND");
					}
					if(m_GTEQ!=null){
						sb.append(" iValue>=");
						sb.append(m_LTEQ);
						sb.append(" AND");
					}
					break;
					
				case org.livee.data.tools.ProfileSchemaProcessor.Schema.SCHEMA_TYPE_FLOAT:
					if(m_EQ!=null){
						sb.append(" fValue=");
						sb.append(m_EQ);
						sb.append(" AND");
					}
					if(m_NEQ!=null){
						sb.append(" fValue!=");
						sb.append(m_NEQ);
						sb.append(" AND");
					}
					if(m_LT!=null){
						sb.append(" fValue<");
						sb.append(m_LT);
						sb.append(" AND");
					}
					if(m_GT!=null){
						sb.append(" fValue>");
						sb.append(m_GT);
						sb.append(" AND");
					}
					if(m_LTEQ!=null){
						sb.append(" fValue<=");
						sb.append(m_LTEQ);
						sb.append(" AND");
					}
					if(m_GTEQ!=null){
						sb.append(" fValue>=");
						sb.append(m_LTEQ);
						sb.append(" AND");
					}
					break;

				case org.livee.data.tools.ProfileSchemaProcessor.Schema.SCHEMA_TYPE_STRING:
					if(m_EQ!=null){
						sb.append(" sValue=\'");
						sb.append(m_EQ);
						sb.append("\' AND");
					}
					if(m_NEQ!=null){
						sb.append(" sValue!=\'");
						sb.append(m_NEQ);
						sb.append("\' AND");
					}
					if(m_LT!=null){
						sb.append(" sValue<\'");
						sb.append(m_LT);
						sb.append("\' AND");
					}
					if(m_GT!=null){
						sb.append(" sValue>\'");
						sb.append(m_GT);
						sb.append("\' AND");
					}
					if(m_LTEQ!=null){
						sb.append(" sValue<=\'");
						sb.append(m_LTEQ);
						sb.append("\' AND");
					}
					if(m_GTEQ!=null){
						sb.append(" sValue>=\'");
						sb.append(m_LTEQ);
						sb.append("\' AND");
					}
					if(m_Contains!=null){
						sb.append(" sValue LIKE \'%");
						sb.append(m_Contains);
						sb.append("%\' AND");
					}
					break;
					
				case org.livee.data.tools.ProfileSchemaProcessor.Schema.SCHEMA_TYPE_TIME:
					if(m_EQ!=null){
						sb.append(" tValue=\'");
						sb.append(m_EQ);
						sb.append("\' AND");
					}
					if(m_NEQ!=null){
						sb.append(" tValue!=\'");
						sb.append(m_NEQ);
						sb.append("\' AND");
					}
					if(m_LT!=null){
						sb.append(" tValue<\'");
						sb.append(m_LT);
						sb.append("\' AND");
					}
					if(m_GT!=null){
						sb.append(" tValue>\'");
						sb.append(m_GT);
						sb.append("\' AND");
					}
					if(m_LTEQ!=null){
						sb.append(" tValue<=\'");
						sb.append(m_LTEQ);
						sb.append("\' AND");
					}
					if(m_GTEQ!=null){
						sb.append(" tValue>=\'");
						sb.append(m_LTEQ);
						sb.append("\' AND");
					}
					break;
					
				case org.livee.data.tools.ProfileSchemaProcessor.Schema.SCHEMA_TYPE_NULL:
				default:	
				}
				int length=sb.length();
				sb.delete(length-4, length);
				sb.append(")");
				return sb.toString();
			}
		}
		return null;
	}
}
