package org.livee.data.replicate;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.livee.data.expr.AbstractData;
import org.livee.data.expr.AbstractParser;
import org.w3c.dom.Element;

/**
 * 
 * @author Hideya Ochiai, The University of Tokyo
 */
public class AggregationReplicationParser implements AbstractParser {

	public String getParserName() {
		return "AggregationReplication";
	}

	public AbstractData parse(Element xml) throws Exception {
		if(xml.getNamespaceURI().equals("http://live-e.org/DataManager/2007/03/")){
			// For Version 200703			
			if(xml.getTagName().equals("aggregation")){
				
				AbstractData agg=DataFactory.getInstance().create("replicateAggregation");

				String id=xml.getAttribute("id");
				String time=xml.getAttribute("time");
				String aggregationType=xml.getAttribute("aggregationType");
				String attrs=xml.getAttribute("attrs");
				String value=xml.getAttribute("value");
				String sValue=xml.getAttribute("sValue");

				if(id!=null && !id.equals("")){
					agg.setAttribute("id", id);
				}
				if(time!=null && !time.equals("")){
					agg.setAttribute("time", time);
				}
				if(aggregationType!=null && !aggregationType.equals("")){
					agg.setAttribute("aggregationType", aggregationType);
				}
				if(attrs!=null && !attrs.equals("")){
					agg.setAttribute("attrs", attrs);
				}
				if(value!=null && !value.equals("")){
					agg.setAttribute("value", value);
				}
				if(sValue!=null && !sValue.equals("")){
					agg.setAttribute("sValue", sValue);
				}
				
				return agg;
				
			}else{
				// Syntax Error
				return null;
			}
		}
		
		return null;
	}

}
