package org.livee.data.replicate;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */


import org.livee.data.expr.AbstractData;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * 
 * @author Hideya Ochiai, The University of Tokyo
 */
public class DataManagerReplicationData implements AbstractData {

	AbstractData m_SensorCombinedCollection;
	AbstractData m_SensorElementCollection;
	AbstractData m_ProfileCombinedArchive;
	AbstractData m_ProfileElementArchive;
	AbstractData m_RawLatestCollection;
	AbstractData m_RawArchiveCollection;
	java.util.ArrayList m_AggregationCollectionCol;
		
	protected DataManagerReplicationData(){
		m_SensorCombinedCollection=null;
		m_SensorElementCollection=null;
		m_ProfileCombinedArchive=null;
		m_ProfileElementArchive=null;
		m_RawLatestCollection=null;
		m_RawArchiveCollection=null;
		m_AggregationCollectionCol=new java.util.ArrayList();
	}

	public void append(AbstractData child) {
		if(child==null){
			return ;
		}
		if(child.getDataType().equals("SensorCombinedCollectionReplication")){
			m_SensorCombinedCollection=child;
		}
		if(child.getDataType().equals("SensorElementCollectionReplication")){
			m_SensorElementCollection=child;
		}
		if(child.getDataType().equals("ProfileCombinedArchiveReplication")){
			m_ProfileCombinedArchive=child;
		}
		if(child.getDataType().equals("ProfileElementArchiveReplication")){
			m_ProfileElementArchive=child;
		}
		if(child.getDataType().equals("RawLatestCollectionReplication")){
			m_RawLatestCollection=child;
		}
		if(child.getDataType().equals("RawArchiveCollectionReplication")){
			m_RawArchiveCollection=child;
		}
		if(child.getDataType().equals("AggregationCollectionReplication")){
			m_AggregationCollectionCol.add(child);	
		}
	}

	public String getAttribute(String name) {
		return null;
	}

	public java.util.Map getAttributes() {
		return null;
	}

	public java.util.Collection getChildren() {
		java.util.ArrayList array=new java.util.ArrayList();
		if(m_SensorCombinedCollection!=null){
			array.add(m_SensorCombinedCollection);
		}
		if(m_SensorElementCollection!=null){
			array.add(m_SensorElementCollection);
		}
		if(m_ProfileCombinedArchive!=null){
			array.add(m_ProfileCombinedArchive);
		}
		if(m_ProfileElementArchive!=null){
			array.add(m_ProfileElementArchive);
		}
		if(m_RawLatestCollection!=null){
			array.add(m_RawLatestCollection);
		}
		if(m_RawArchiveCollection!=null){
			array.add(m_RawArchiveCollection);
		}
		array.addAll(m_AggregationCollectionCol);
		
		return array;
	}

	public String getDataType() {
		return "DataManagerReplication";
	}

	public String getTextContent() {
		return null;
	}

	public Element getXml(Document doc) {
		
		String ns="http://live-e.org/DataManager/2007/03/";
		Element dataManagerReplication=doc.createElementNS(ns, "dataManagerReplication");
		
		if(m_SensorCombinedCollection!=null){
			dataManagerReplication.appendChild(m_SensorCombinedCollection.getXml(doc));
		}
		if(m_SensorElementCollection!=null){
			dataManagerReplication.appendChild(m_SensorElementCollection.getXml(doc));
		}
		if(m_ProfileCombinedArchive!=null){
			dataManagerReplication.appendChild(m_ProfileCombinedArchive.getXml(doc));
		}
		if(m_ProfileElementArchive!=null){
			dataManagerReplication.appendChild(m_ProfileElementArchive.getXml(doc));
		}
		if(m_RawLatestCollection!=null){
			dataManagerReplication.appendChild(m_RawLatestCollection.getXml(doc));
		}
		if(m_RawArchiveCollection!=null){
			dataManagerReplication.appendChild(m_RawArchiveCollection.getXml(doc));
		}
		java.util.Iterator itr=m_AggregationCollectionCol.iterator();
		while(itr.hasNext()){
			AbstractData child=(AbstractData)itr.next();
			dataManagerReplication.appendChild(child.getXml(doc));
		}
		return dataManagerReplication;
	}

	public void remove(AbstractData child) {}

	public void removeAttribute(String name) {}

	public void setAttribute(String name, String value) {}

	public void setTextContent(String text) {}

}
