package org.livee.data.replicate;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.db.IDbCommand;
import org.livee.data.exception.LiveEException;
import org.livee.data.expr.AbstractData;
import org.livee.util.W3CTimestamp;

/**
 * 
 * @author Hideya Ochiai, The University of Tokyo
 */
public class GetReplicateProfileCombinedArchiveCommand implements IDbCommand {
	
	String m_Start;
	String m_End;
	AbstractData m_ProfileCombinedCol;
	boolean m_bSucceeded;
	
	public GetReplicateProfileCombinedArchiveCommand(String start,String end){
		m_Start=start;
		m_End=end;
		m_bSucceeded=false;
	}
	
	boolean succeeded(){
		return m_bSucceeded;
	}
	
	public AbstractData getProfileCombinedArchive(){
		return m_ProfileCombinedCol;
	}

	
	public void execCommand(Connection connection, Map conf) throws Exception {
	
		java.sql.Statement stmt=null;
		DataFactory df=DataFactory.getInstance();
		m_ProfileCombinedCol=df.create("replicateProfileCombinedArchive");
		try{
			stmt=connection.createStatement();
			
			String sql;
			if(m_Start==null){
				if(m_End==null){
					sql="SELECT id,created,expired,latitude,longitude,attrs FROM profile.combined_archive;";
				}else{
					sql="SELECT id,created,expired,latitude,longitude,attrs FROM profile.combined_archive WHERE created<\'"+m_End+"\'";
				}
			}else{
				if(m_End==null){
					sql="SELECT id,created,expired,latitude,longitude,attrs FROM profile.combined_archive WHERE created>=\'"+m_Start+"\';";
				}else{
					sql="SELECT id,created,expired,latitude,longitude,attrs FROM profile.combined_archive WHERE created>=\'"+m_Start+"\' AND created<\'"+m_End+"\'";					
				}
			}
			java.sql.ResultSet rset=stmt.executeQuery(sql);
			
			while(rset.next()){
				String id=rset.getString(1);
				String created=W3CTimestamp.toString(rset.getTimestamp(2),java.util.TimeZone.getDefault());
				String expired=W3CTimestamp.toString(rset.getTimestamp(3),java.util.TimeZone.getDefault());
				String latitude=rset.getString(4);
				String longitude=rset.getString(5);
				String attrs=rset.getString(6);
				
				AbstractData profileCombined=df.create("replicateProfileCombined");
				profileCombined.setAttribute("id", id);
				profileCombined.setAttribute("created", created);
				profileCombined.setAttribute("expired", expired);
				profileCombined.setAttribute("latitude", latitude);
				profileCombined.setAttribute("longitude", longitude);
				profileCombined.setAttribute("attrs", attrs);
				
				m_ProfileCombinedCol.append(profileCombined);
			}
			
			m_bSucceeded=true;
		}
		catch(Exception e){
			throw e;
		}
		finally{
			stmt.close();
		}
	}


	public String getCommandName() {
		return "GetReplicateProfileCombinedArchive";
	}

	public short getImplVersion() {
		return 0x0100;
	}

}
