package org.livee.data.replicate;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.util.Map;
import org.livee.data.expr.AbstractParser;

/**
 * Dymanic Parser Provider which uses org.livee.data.expr.ParserFactory
 * @author  Hideya Ochiai, The University of Tokyo
 * @version  1.0
 */
public class ParserFactory {

	static ParserFactory m_Singleton;
	Map m_ParserMap;
	
	/**
	 * Create this instance and prepare for Parser Provider
	 */
	ParserFactory(){
		m_ParserMap=new java.util.Hashtable();
		
		m_ParserMap.put("replicateDataManager",new DataManagerReplicationParser());
		m_ParserMap.put("replicateSensorCombinedCollection",new SensorCombinedCollectionReplicationParser());
		m_ParserMap.put("replicateSensorCombined",new SensorCombinedReplicationParser());
		m_ParserMap.put("replicateSensorElementCollection",new SensorElementCollectionReplicationParser());
		m_ParserMap.put("replicateSensorElement",new SensorElementReplicationParser());
		m_ParserMap.put("replicateProfileCombinedArchive",new ProfileCombinedArchiveReplicationParser());
		m_ParserMap.put("replicateProfileCombined",new ProfileCombinedReplicationParser());
		m_ParserMap.put("replicateProfileElementArchive",new ProfileElementArchiveReplicationParser());
		m_ParserMap.put("replicateProfileElement",new ProfileElementReplicationParser());
		m_ParserMap.put("replicateRawLatestCollection",new RawLatestCollectionReplicationParser());
		m_ParserMap.put("replicateRawLatest",new RawLatestReplicationParser());
		m_ParserMap.put("replicateRawArchiveCollection",new RawArchiveCollectionReplicationParser());
		m_ParserMap.put("replicateRawArchive",new RawArchiveReplicationParser());
		m_ParserMap.put("replicateAggregationCollection",new AggregationCollectionReplicationParser());
		m_ParserMap.put("replicateAggregation",new AggregationReplicationParser());

		m_Singleton=this;
	}
	
	/**
	 * @return get the Singleton
	 */
	public static ParserFactory getInstance(){
		if(m_Singleton==null){
			new ParserFactory();
		}
		return m_Singleton;
	}
	
	/**
	 * @param name Specify Parser Type
	 * @return Parser or null
	 */
	public AbstractParser get(String name){
		
		Object o=m_ParserMap.get(name);
		if(o!=null){
			return (AbstractParser)o;
		}
		return org.livee.data.expr.ParserFactory.getInstance().get(name);
	}

}
