package org.livee.data.replicate;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.livee.data.expr.AbstractData;
import org.livee.data.expr.AbstractParser;
import org.livee.data.replicate.DataFactory;
import org.w3c.dom.Element;

/**
 * 
 * @author Hideya Ochiai, The University of Tokyo
 */
public class ProfileCombinedReplicationParser implements AbstractParser {

	public String getParserName() {
		return "ProfileCombinedReplication";
	}

	public AbstractData parse(Element xml) throws Exception {
		
		if(xml.getNamespaceURI().equals("http://live-e.org/DataManager/2007/03/")){
			// For Version 200703			
			if(xml.getTagName().equals("profileCombined")){
				
				AbstractData profileCombined=DataFactory.getInstance().create("replicateProfileCombined");

				String id=xml.getAttribute("id");
				String created=xml.getAttribute("created");
				String expired=xml.getAttribute("expired");
				String latitude=xml.getAttribute("latitude");
				String longitude=xml.getAttribute("longitude");
				String attrs=xml.getAttribute("attrs");

				if(id!=null && !id.equals("")){
					profileCombined.setAttribute("id", id);
				}
				if(created!=null && !created.equals("")){
					profileCombined.setAttribute("created", created);
				}
				if(expired!=null && !expired.equals("")){
					profileCombined.setAttribute("expired", expired);
				}
				if(latitude!=null && !latitude.equals("")){
					profileCombined.setAttribute("latitude", latitude);
				}
				if(longitude!=null && !longitude.equals("")){
					profileCombined.setAttribute("longitude", longitude);
				}
				if(attrs!=null && !attrs.equals("")){
					profileCombined.setAttribute("attrs", attrs);
				}
				
				return profileCombined;
				
			}else{
				// Syntax Error
				return null;
			}
		}
		
		return null;
	}

}
