package org.livee.data.replicate;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.db.IDbCommand;
import org.livee.data.expr.AbstractData;

/**
 * 
 * @author Hideya Ochiai, The University of Tokyo
 */
public class RestoreArchiveDataCommand implements IDbCommand {

	boolean m_bSucceeded;
	AbstractData m_DataManagerReplication;
	
	public RestoreArchiveDataCommand(AbstractData dataManager){
		m_bSucceeded=false;
		m_DataManagerReplication=dataManager;
	}
	
	public boolean succeeded(){
		return m_bSucceeded;
	}
	
	public void execCommand(Connection connection, Map conf) throws Exception {
		
		java.util.Iterator itr=m_DataManagerReplication.getChildren().iterator();
		
		m_bSucceeded=true;
		while(itr.hasNext()){
			AbstractData col=(AbstractData)itr.next();
			if(col.getDataType().equals("RawArchiveCollectionReplication")){
				SetReplicateRawArchiveCollectionCommand rawArchiveCollection_command=new SetReplicateRawArchiveCollectionCommand(col);
				rawArchiveCollection_command.execCommand(connection, conf);
				if(!rawArchiveCollection_command.succeeded()){
					m_bSucceeded=false;
				}
			}else if(col.getDataType().equals("AggregationCollectionReplication")){
				String scope=col.getAttribute("scope");
				if(scope!=null){
					if(scope.equals("hour")){
						SetReplicateAggHourCollectionCommand command=new SetReplicateAggHourCollectionCommand(col);
						command.execCommand(connection, conf);
						if(!command.succeeded()){
							m_bSucceeded=false;
						}
					}else if(scope.equals("day")){
						SetReplicateAggDayCollectionCommand command=new SetReplicateAggDayCollectionCommand(col);
						command.execCommand(connection, conf);
						if(!command.succeeded()){
							m_bSucceeded=false;
						}
					}else if(scope.equals("month")){
						SetReplicateAggMonthCollectionCommand command=new SetReplicateAggMonthCollectionCommand(col);
						command.execCommand(connection, conf);
						if(!command.succeeded()){
							m_bSucceeded=false;
						}						
					}
				}
			}
		}
	}

	public String getCommandName() {
		return "RestoreArchiveData";
	}

	public short getImplVersion() {
		return 0x0100;
	}

}
