package org.livee.data.replicate;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.db.IDbCommand;
import org.livee.data.expr.AbstractData;
import org.livee.data.replicate.DataFactory;

/**
 * 
 * @author Hideya Ochiai, The University of Tokyo
 */
public class RestoreProfileCommand implements IDbCommand {

	boolean m_bSucceeded;
	AbstractData m_DataManagerReplication;
	
	public RestoreProfileCommand(AbstractData dataManager){
		m_bSucceeded=false;
		m_DataManagerReplication=dataManager;
	}
	
	public boolean succeeded(){
		return m_bSucceeded;
	}
	

	public void execCommand(Connection connection, Map conf) throws Exception {
		
		java.util.Iterator itr=m_DataManagerReplication.getChildren().iterator();
		
		boolean sc=false;
		boolean se=false;
		boolean pc=false;
		boolean pe=false;
		
		while(itr.hasNext()){
			AbstractData data=(AbstractData)itr.next();
			if(data==null){
				continue;
			}
			if(data.getDataType().equals("SensorCombinedCollectionReplication")){
				SetReplicateSensorCombinedCollectionCommand command=new SetReplicateSensorCombinedCollectionCommand(data);
				command.execCommand(connection, conf);
				sc=command.succeeded();
			}else if(data.getDataType().equals("SensorElementCollectionReplication")){
				SetReplicateSensorElementCollectionCommand command=new SetReplicateSensorElementCollectionCommand(data);
				command.execCommand(connection, conf);
				se=command.succeeded();
			}else if(data.getDataType().equals("ProfileCombinedArchiveReplication")){
				SetReplicateProfileCombinedArchiveCommand command=new SetReplicateProfileCombinedArchiveCommand(data);
				command.execCommand(connection, conf);
				pc=command.succeeded();
			}else if(data.getDataType().equals("ProfileElementArchiveReplication")){
				SetReplicateProfileElementArchiveCommand command=new SetReplicateProfileElementArchiveCommand(data);
				command.execCommand(connection, conf);
				pe=command.succeeded();
			}
		}
		
		if(sc && se && pc && pe){
			m_bSucceeded=true;
		}
	}

	public String getCommandName() {
		return "RestoreProfile";
	}

	public short getImplVersion() {
		return 0x0100;
	}

}
