package org.livee.data.replicate;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.db.IDbCommand;
import org.livee.data.expr.AbstractData;

/**
 * 
 * @author Hideya Ochiai, The University of Tokyo
 */
public class SetReplicateAggMonthCollectionCommand implements IDbCommand {


	AbstractData m_AggregationCol;
	boolean m_bSucceeded;
	
	public SetReplicateAggMonthCollectionCommand(AbstractData aggCol){
		m_AggregationCol=aggCol;
		m_bSucceeded=false;
	}
	
	boolean succeeded(){
		return m_bSucceeded;
	}
	
	
	public void execCommand(Connection connection, Map conf) throws Exception {
		
		java.sql.Statement stmt=null;
		
		try{
			stmt=connection.createStatement();
			
			java.util.Iterator itr=m_AggregationCol.getChildren().iterator();
			while(itr.hasNext()){
				AbstractData agg=(AbstractData)itr.next();
				if(agg==null){ continue; }
				String id=agg.getAttribute("id");
				String time=agg.getAttribute("time");
				String aggregationType=agg.getAttribute("aggregationType");
				String attrs=agg.getAttribute("attrs");
				String value=agg.getAttribute("value");
				String sValue=agg.getAttribute("sValue");
				
				if(id==null || time==null){
					continue;
				}
				
				StringBuilder insert=new StringBuilder("INSERT INTO aggregation.month (id,time,aggregationType,attrs,value,sValue) VALUES (\'");
				insert.append(id);
				insert.append("\',\'");
				insert.append(time);
				
				if(aggregationType!=null){
					insert.append("\',\'");
					insert.append(aggregationType);
				}else{
					insert.append("\',null");
				}

				if(attrs!=null){
					insert.append("\',\'");
					insert.append(attrs);
					insert.append("\',");
				}else{
					insert.append("\',\'{}\',");
				}
				
				if(value!=null){
					insert.append(value);
				}else{
					insert.append("null");
				}
				
				if(sValue!=null){
					insert.append(",\'");
					insert.append(sValue);
					insert.append("\');");
				}else{
					insert.append(",null);");
				}

				String sql=insert.toString();
				try{
					stmt.execute(sql);
				}
				catch(Exception e){
					//System.err.println(sql);
					//e.printStackTrace();
				}
			}
			m_bSucceeded=true;
		}
		catch(Exception e){
			throw e;
		}
		finally{
			stmt.close();
		}

	}

	public String getCommandName() {
		return "SetReplicateAggMonthCollection";
	}

	public short getImplVersion() {
		return 0x0100;
	}


}
