package org.livee.data.srv;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.livee.data.exception.InternalLiveEException;
import org.livee.data.exception.LiveEException;
import org.livee.data.expr.AbstractData;
import org.livee.data.replicate.DumpProfileCommand;
import org.livee.data.replicate.DumpArchiveDataCommand;
import org.livee.data.replicate.DumpLatestDataCommand;
import org.livee.data.tools.LogManager;
import org.livee.util.Xml;
import org.w3c.dom.Document;

/**
 * 
 * @author Hideya Ochiai, The University of Tokyo
 */
public class DataManagerReplication200703 {

	public String replicateProfile(String timeStart,String timeEnd){
	
		LogManager.postNormal("service.DataManagerReplication200703.replicateProfile.begin",null);

		org.livee.data.db.DatabaseAdapter db=org.livee.data.db.DatabaseAdapter.getInstance();

		try{
			DumpProfileCommand command=new DumpProfileCommand(timeStart,timeEnd);
			db.postCommand(command);
			if(command.succeeded()){
				AbstractData data=command.getDataManagerReplication();
		
				Document doc=Xml.newDocument();
				String retXml=Xml.toString(data.getXml(doc));
				LogManager.postNormal("service.DataManagerReplication200703.replicateProfile.end",null);				
				return retXml;
			}
		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.DataManagerReplication200703.replicateProfile.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.DataManagerReplication200703.replicateProfile.error", e.getMessage());
			return exp.getMessage();
		}

		// return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.DataManagerReplication200703.replicateProfile.error", "unknown");
		return exp.getMessage();

	}
	
	public String replicateLatestData(){
		
		LogManager.postNormal("service.DataManagerReplication200703.replicateLatestData.begin",null);

		org.livee.data.db.DatabaseAdapter db=org.livee.data.db.DatabaseAdapter.getInstance();

		try{
			DumpLatestDataCommand command=new DumpLatestDataCommand();
			db.postCommand(command);
			if(command.succeeded()){
				AbstractData data=command.getDataManagerReplication();
		
				Document doc=Xml.newDocument();
				String retXml=Xml.toString(data.getXml(doc));
				LogManager.postNormal("service.DataManagerReplication200703.replicateLatestData.end",null);
				
				return retXml;
			}
		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.DataManagerReplication200703.replicateLatestData.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.DataManagerReplication200703.replicateLatestData.error", e.getMessage());
			return exp.getMessage();
		}

		// return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.DataManagerReplication200703.replicateLatestData.error", "unknown");
		return exp.getMessage();

	}
	
	public String replicateArchiveData(String timeStart,String timeEnd){

		LogManager.postNormal("service.DataManagerReplication200703.replicateArchiveData.begin",null);
		
		org.livee.data.db.DatabaseAdapter db=org.livee.data.db.DatabaseAdapter.getInstance();

		try{
			DumpArchiveDataCommand command=new DumpArchiveDataCommand(timeStart,timeEnd);
			db.postCommand(command);
			if(command.succeeded()){
				AbstractData data=command.getDataManagerReplication();
		
				Document doc=Xml.newDocument();
				String retXml=Xml.toString(data.getXml(doc));
				LogManager.postNormal("service.DataManagerReplication200703.replicateArchiveData.end",null);
				return retXml;
			}
		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.DataManagerReplication200703.replicateArchiveData.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.DataManagerReplication200703.replicateArchiveData.error", e.getMessage());
			return exp.getMessage();
		}

		// return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.DataManagerReplication200703.replicateArchiveData.error", "unknown");
		return exp.getMessage();

	}
}
