package org.livee.data.srv;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.livee.data.exception.InternalLiveEException;
import org.livee.data.exception.LiveEException;
import org.livee.data.expr.AbstractData;
import org.livee.data.db.DatabaseAdapter;
import org.livee.data.db.CommandFactory;
import org.livee.data.db.GetArchiveCombinedDataCommand;
import org.livee.data.db.GetLatestDataAllCommand;
import org.livee.data.db.GetLatestDataCommand;
import org.livee.data.db.GetLatestDataByAreaRectCommand;
import org.livee.data.db.GetViewProfileAllCommand;
import org.livee.data.db.GetViewProfileCommand;
import org.livee.data.db.GetViewProfileArchiveCommand;
import org.livee.data.db.GetViewProfileByAreaRectCommand;
import org.livee.data.db.GetAvailableLocalesCommand;
import org.livee.data.db.GetProfileSchemaCommand;
import org.livee.data.db.GetAggHourCommand;
import org.livee.data.db.GetAggDayCommand;
import org.livee.data.db.GetAggMonthCommand;
import java.util.Calendar;
import java.util.TimeZone;

import org.livee.data.expr.DataFactory;
import org.livee.data.tools.LogManager;

import org.livee.util.Xml;
import org.livee.util.W3CTimestamp;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * 
 * @author Hideya Ochiai, The University of Tokyo
 */
public class DataProvider200703 {

	
	public String getProfileSchema(){
		
		LogManager.postNormal("service.DataProvider200703.getProfileSchema.begin",null);
		
		try{
			DatabaseAdapter db=DatabaseAdapter.getInstance();
			GetProfileSchemaCommand command=CommandFactory.createGetProfileSchemaCommand();
 
			db.postCommand(command);

			if(command.succeeded()){
				Document doc=Xml.newDocument();
				AbstractData data=command.getProfileSchema();
				Element root=data.getXml(doc);
				String retXml=Xml.toString(root);
				LogManager.postNormal("service.DataProvider200703.getProfileSchema.end",null);
				return retXml;
			}
		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.DataProvider200703.getProfileSchema.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.DataProvider200703.getProfileSchema.error", e.getMessage());
			return exp.getMessage();
		}

		// return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.DataProvider200703.getProfileSchema.error", "unknown");
		return exp.getMessage();

	}
	
	/*
	public String getAvailableLocales(){
		
		DatabaseAdapter db=DatabaseAdapter.getInstance();
		GetAvailableLocalesCommand command=CommandFactory.createGetAvailableLocalesCommand();
	
		try{
			db.postCommand(command);
		}
		catch(Exception e){
			e.printStackTrace();
		}
		
		if(command.succeeded()){
			Document doc=Xml.newDocument();
			AbstractData data=command.getAvailableLocales();
			Element root=data.getXml(doc);
			return Xml.toString(root);
		}
		
		// return error code
		return "<error />";
	}
	
	public String getAvailableTimeZones(){
		
		DataFactory df=DataFactory.getInstance();
		AbstractData timezones=df.create("TimeZoneCollection");
		
		String[] ids=TimeZone.getAvailableIDs();
		java.util.Arrays.sort(ids);
		for(int i=0;i<ids.length;i++){
			AbstractData tz=df.create("TimeZone");
			timezones.append(tz);
			tz.setTextContent(ids[i]);
		}
		
		Document doc=Xml.newDocument();
		Element root=timezones.getXml(doc);
		return Xml.toString(root);
	}
	*/
	
	
	public String getLatestDataAll(String timezone,String locale){
	
		LogManager.postNormal("service.DataProvider200703.getLatestDataAll.begin",null);
		
		try{
			TimeZone tz=TimeZone.getTimeZone(timezone);
			
			DatabaseAdapter db=DatabaseAdapter.getInstance();
			GetLatestDataAllCommand command=new GetLatestDataAllCommand(tz,locale);

			db.postCommand(command);
		
			if(command.succeeded()){
				Document doc=Xml.newDocument();
				AbstractData data=command.getCollection();
				Element root=data.getXml(doc);
				
				String retXml=Xml.toString(root);
				LogManager.postNormal("service.DataProvider200703.getLatestDataAll.end",null);
				return retXml;
			}

		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.DataProvider200703.getLatestDataAll.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.DataProvider200703.getLatestDataAll.error", e.getMessage());
			return exp.getMessage();
		}

		// return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.DataProvider200703.getLatestDataAll.error", "unknown");
		return exp.getMessage();
	}
	
	public String getLatestData(String id,String timezone,String locale){
		
		LogManager.postNormal("service.DataProvider200703.getLatestData.begin",null);
		
		try{
			TimeZone tz=TimeZone.getTimeZone(timezone);
			
			DatabaseAdapter db=DatabaseAdapter.getInstance();
			GetLatestDataCommand command=new GetLatestDataCommand(id,tz,locale);

			db.postCommand(command);
			if(command.succeeded()){
				Document doc=Xml.newDocument();
				AbstractData data=command.getCombinedSensor();
				Element root=data.getXml(doc);
				String retXml=Xml.toString(root);
				LogManager.postNormal("service.DataProvider200703.getLatestData.end",null);
				
				return retXml;
			}
		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.DataProvider200703.getLatestData.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.DataProvider200703.getLatestData.error", e.getMessage());
			return exp.getMessage();
		}

		// return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.DataProvider200703.getLatestData.error", "unknown");
		return exp.getMessage();

	}
	
	public String getLatestDataByAreaRect(double north,double south,double east,double west,String timezone,String locale){
		
		LogManager.postNormal("service.DataProvider200703.getLatestDataByAreaRect.begin",null);
		
		try{			
			TimeZone tz=TimeZone.getTimeZone(timezone);
			
			DatabaseAdapter db=DatabaseAdapter.getInstance();
			GetLatestDataByAreaRectCommand command=new GetLatestDataByAreaRectCommand(north,south,east,west,tz,locale);

			db.postCommand(command);
			if(command.succeeded()){
				Document doc=Xml.newDocument();
				AbstractData data=command.getCollection();
				Element root=data.getXml(doc);
				
				String retXml=Xml.toString(root);
				LogManager.postNormal("service.DataProvider200703.getLatestDataByAreaRect.end",null);
				return retXml;
			}
		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.DataProvider200703.getLatestDataByAreaRect.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.DataProvider200703.getLatestDataByAreaRect.error", e.getMessage());
			return exp.getMessage();
		}

		// return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.DataProvider200703.getLatestDataByAreaRect.error", "unknown");
		return exp.getMessage();

	}
	
	public String getDataHourlyAggregated(String id,String start,String end,String timezone,String locale){

		LogManager.postNormal("service.DataProvider200703.getDataHourlyAggregated.begin",null);
		
		try{
			TimeZone tz=TimeZone.getTimeZone(timezone);
			
			DatabaseAdapter db=DatabaseAdapter.getInstance();
			GetAggHourCommand command=new GetAggHourCommand(id,start,end,tz,locale);

			db.postCommand(command);
		
			if(command.succeeded()){
				Document doc=Xml.newDocument();
				AbstractData data=command.getCombinedSensor();
				Element root=data.getXml(doc);
				
				String retXml=Xml.toString(root);
				LogManager.postNormal("service.DataProvider200703.getDataHourlyAggregated.end",null);
				return retXml;
			}
		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.DataProvider200703.getDataHourlyAggregated.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.DataProvider200703.getDataHourlyAggregated.error", e.getMessage());
			return exp.getMessage();
		}

		// return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.DataProvider200703.getDataHourlyAggregated.error", "unknown");
		return exp.getMessage();

	}
	
	public String getDataDailyAggregated(String id,String start,String end,String timezone,String locale){

		LogManager.postNormal("service.DataProvider200703.getDataDailyAggregated.begin",null);
		
		try{
			TimeZone tz=TimeZone.getTimeZone(timezone);

			DatabaseAdapter db=DatabaseAdapter.getInstance();
			GetAggDayCommand command=new GetAggDayCommand(id,start,end,tz,locale);
			
			db.postCommand(command);

			if(command.succeeded()){
				Document doc=Xml.newDocument();
				AbstractData data=command.getCombinedSensor();
				Element root=data.getXml(doc);
				String retXml=Xml.toString(root);
				
				LogManager.postNormal("service.DataProvider200703.getDataDailyAggregated.end",null);
				return retXml;
			}

		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.DataProvider200703.getDataDailyAggregated.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.DataProvider200703.getDataDailyAggregated.error", e.getMessage());
			return exp.getMessage();
		}

		// return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.DataProvider200703.getDataDailyAggregaed.error", "unknown");
		return exp.getMessage();
	}

	public String getDataMonthlyAggregated(String id,String start,String end,String timezone,String locale){

		LogManager.postNormal("service.DataProvider200703.getDataMonthlyAggregated.begin",null);
	
		try{
			TimeZone tz=TimeZone.getTimeZone(timezone);

			DatabaseAdapter db=DatabaseAdapter.getInstance();
			GetAggMonthCommand command=new GetAggMonthCommand(id,start,end,tz,locale);

			db.postCommand(command);

			if(command.succeeded()){
				Document doc=Xml.newDocument();
				AbstractData data=command.getCombinedSensor();
				Element root=data.getXml(doc);

				LogManager.postNormal("service.DataProvider200703.getDataMonthlyAggregated.end",null);
				return Xml.toString(root);
			}
		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.DataProvider200703.getDataMonthlyAggregated.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.DataProvider200703.getDataMonthlyAggregated.error", e.getMessage());
			return exp.getMessage();
		}

		// return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.DataProvider200703.getDataMonthlyAggregated.error", "unknown");
		return exp.getMessage();
	}

	public String getProfileAll(String timezone,String locale){
		
		LogManager.postNormal("service.DataProvider200703.getProfileAll.begin",null);
			
		try{
			TimeZone tz=TimeZone.getTimeZone(timezone);
			
			DatabaseAdapter db=DatabaseAdapter.getInstance();
			GetViewProfileAllCommand command=new GetViewProfileAllCommand(tz,locale);

			db.postCommand(command);
			if(command.succeeded()){
				Document doc=Xml.newDocument();
				AbstractData data=command.getCollection();
				Element root=data.getXml(doc);
				String retXml=Xml.toString(root);
				LogManager.postNormal("service.DataProvider200703.getProfileAll.end",null);
				return retXml;
			}
		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.DataProvider200703.getProfileAll.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.DataProvider200703.getProfileAll.error", e.getMessage());
			return exp.getMessage();
		}

		// return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.DataProvider200703.getProfileAll.error", "unknown");
		return exp.getMessage();
	}
	
	public String getProfile(String id,String timezone,String locale){
		
		LogManager.postNormal("service.DataProvider200703.getProfile.begin",null);
		
		try{
			TimeZone tz=TimeZone.getTimeZone(timezone);
			
			DatabaseAdapter db=DatabaseAdapter.getInstance();
			GetViewProfileCommand command=new GetViewProfileCommand(id,tz,locale);

			db.postCommand(command);
			if(command.succeeded()){
				Document doc=Xml.newDocument();
				AbstractData data=command.getCombinedSensor();
				Element root=data.getXml(doc);
				
				String retXml=Xml.toString(root);
				LogManager.postNormal("service.DataProvider200703.getProfile.end",null);
				return retXml;
			}

		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.DataProvider200703.getProfile.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.DataProvider200703.getProfile.error", e.getMessage());
			return exp.getMessage();
		}

		// return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.DataProvider200703.getProfile.error", "unknown");
		return exp.getMessage();
	}
		
	public String getProfileByAreaRect(double north,double south,double east,double west,String timezone,String locale){
		
		LogManager.postNormal("service.DataProvider200703.getProfileByAreaRect.begin",null);
		
	
		try{
			TimeZone tz=TimeZone.getTimeZone(timezone);
			
			DatabaseAdapter db=DatabaseAdapter.getInstance();
			GetViewProfileByAreaRectCommand command=new GetViewProfileByAreaRectCommand(north,south,east,west,tz,locale);

			db.postCommand(command);

			if(command.succeeded()){
				Document doc=Xml.newDocument();
				AbstractData data=command.getCollection();
				Element root=data.getXml(doc);
				
				String retXml=Xml.toString(root);
				LogManager.postNormal("service.DataProvider200703.getProfileByAreaRect.end",null);
				return retXml;
			}

		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.DataProvider200703.getProfileByAreaRect.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.DataProvider200703.getProfileByAreaRect.error", e.getMessage());
			return exp.getMessage();
		}

		// return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.DataProvider200703.getProfileByAreaRect.error", "unknown");
		return exp.getMessage();
	}
	
	public String getArchiveCombinedData(String id,String start,String end,String timezone,String locale){
		
		LogManager.postNormal("service.DataProvider200703.getArchiveCombinedData.begin",null);
		
		try{
			TimeZone tz=TimeZone.getTimeZone(timezone);

			DatabaseAdapter db=DatabaseAdapter.getInstance();
			GetArchiveCombinedDataCommand command=new GetArchiveCombinedDataCommand(id,start,end,tz,locale);

			db.postCommand(command);
			if(command.succeeded()){
				Document doc=Xml.newDocument();
				AbstractData data=command.getCombinedSensor();
				Element root=data.getXml(doc);
				String retXml=Xml.toString(root);
				LogManager.postNormal("service.DataProvider200703.getArchiveCombinedData.end",null);
				return retXml;
			}
		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.DataProvider200703.getArchiveCombinedData.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.DataProvider200703.getArchiveCombinedData.error", e.getMessage());
			return exp.getMessage();
		}

		// return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.DataProvider200703.getArchiveCombinedData.error", "unknown");
		return exp.getMessage();
	}
	
	public String getArchiveProfile(String id,String start,String end,String timezone,String locale){
		
		LogManager.postNormal("service.DataProvider200703.getArchiveProfile.begin",null);
		
		try{
			TimeZone tz=TimeZone.getTimeZone(timezone);
			Calendar c_start=null;
			Calendar c_end=null;
 			c_start=W3CTimestamp.parse(start);
			c_end=W3CTimestamp.parse(end);

			DatabaseAdapter db=DatabaseAdapter.getInstance();
			GetViewProfileArchiveCommand command=new GetViewProfileArchiveCommand(id,c_start,c_end,tz,locale);

			db.postCommand(command);
			if(command.succeeded()){
				Document doc=Xml.newDocument();
				AbstractData data=command.getCollection();
				Element root=data.getXml(doc);
				String retXml=Xml.toString(root);
				LogManager.postNormal("service.DataProvider200703.getArchiveProfile.end",null);
				return retXml;
			}
		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.DataProvider200703.getArchiveProfile.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.DataProvider200703.getArchiveProfile.error", e.getMessage());
			return exp.getMessage();
		}

		// return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.DataProvider200703.getArchiveProfile.error", "unknown");
		return exp.getMessage();

	}
	
}
