package org.livee.data.srv;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.livee.util.*;
import org.livee.data.db.*;
import org.livee.data.exception.InternalLiveEException;
import org.livee.data.exception.LiveEException;
import org.livee.data.expr.*;
import org.livee.data.tools.LogManager;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * 
 * @author Hideya Ochiai, The University of Tokyo
 */
public class ProfileManagement200703 {

	public String getAvailableLocales(){
	
		LogManager.postNormal("service.ProfileManagement200703.getAvailableLocales.begin",null);
	
		try{
			DatabaseAdapter db=DatabaseAdapter.getInstance();
			GetAdminLocaleSchemaCommand command=CommandFactory.createGetAdminLocaleSchemaCommand();

			db.postCommand(command);

			if(command.succeeded()){
				Document doc=Xml.newDocument();
				AbstractData data=command.getLocaleSchema();
				Element root=data.getXml(doc);
				
				String retXml=Xml.toString(root);
				LogManager.postNormal("service.ProfileManagement200703.getAvailableLocales.end",null);
				return retXml;
			}
		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.ProfileManagement200703.getAvailableLocales.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.ProfileManagement200703.getAvailableLocales.error", e.getMessage());
			return exp.getMessage();
		}

		// return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.ProfileManagement200703.getAvailableLocales.error", "unknown");
		return exp.getMessage();

	}

	public String getProfileSchema(){
		
		LogManager.postNormal("service.ProfileManagement200703.getProfileSchema.begin",null);
	
		try{
			DatabaseAdapter db=DatabaseAdapter.getInstance();
			GetAdminProfileSchemaCommand command=CommandFactory.createGetAdminProfileSchemaCommand();

			db.postCommand(command);

			if(command.succeeded()){
				Document doc=Xml.newDocument();
				AbstractData data=command.getProfileSchema();
				Element root=data.getXml(doc);

				String retXml=Xml.toString(root);
				LogManager.postNormal("service.ProfileManagement200703.getProfileSchema.end",null);
				return retXml;
			}
		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.ProfileManagement200703.getProfileSchema.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.ProfileManagement200703.getProfileSchema.error", e.getMessage());
			return exp.getMessage();
		}

		// return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.ProfileManagement200703.getProfileSchema.error", "unknown");
		return exp.getMessage();
		
	}

	public String getProfile(String id) {

		LogManager.postNormal("service.ProfileManagement200703.getProfile.begin",null);
		
		try {
			DatabaseAdapter db = DatabaseAdapter.getInstance();
			GetProfileCommand command = new GetProfileCommand(id);
			
			db.postCommand(command);
			
			if ( command.succeeded()) {
				org.w3c.dom.Document doc = org.livee.util.Xml.newDocument();
				org.livee.data.expr.AbstractData data = command.getResult();
				org.w3c.dom.Element el = data.getXml(doc);
				String retXml = Xml.toString(el);
				LogManager.postNormal("service.ProfileManagement200703.getProfile.end",null);
				return retXml;
			}
		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.ProfileManagement200703.getProfile.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.ProfileManagement200703.getProfile.error", e.getMessage());
			return exp.getMessage();
		}

		// return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.ProfileManagement200703.getProfile.error", "unknown");
		return exp.getMessage();

	}

	public String getProfileAll(String timezone, String locale) {
		
		LogManager.postNormal("service.ProfileManagement200703.getProfileAll.begin",null);
		
		try {
			DatabaseAdapter db = DatabaseAdapter.getInstance();
			
			GetViewProfileAllCommand command = new GetViewProfileAllCommand(java.util.TimeZone.getTimeZone(timezone), locale);
			db.postCommand(command);
			
			if(command.succeeded()){
				org.livee.data.expr.AbstractData data = command.getCollection();

				org.w3c.dom.Document doc = org.livee.util.Xml.newDocument();
				org.w3c.dom.Element el = data.getXml(doc);
				String retXml = org.livee.util.Xml.toString(el);
				
				LogManager.postNormal("service.ProfileManagement200703.getProfileAll.end",null);
				return retXml;
			}
		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.ProfileManagement200703.getProfileAll.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.ProfileManagement200703.getProfileAll.error", e.getMessage());
			return exp.getMessage();
		}

		// return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.ProfileManagement200703.getProfileAll.error", "unknown");
		return exp.getMessage();

	}

	public String setProfile(String profile) {
		
		LogManager.postNormal("service.ProfileManagement200703.setProfile.begin",null);

		try{
			ParserFactory pf=ParserFactory.getInstance();
			AbstractParser combined_parser=pf.get("combined");

			org.w3c.dom.Element xml_profile=Xml.parseXml(profile).getDocumentElement();
			AbstractData combined=combined_parser.parse(xml_profile);
			
			String passwd=Base64Coder.decodeString(combined.getAttribute("authorization"));
			
			combined.removeAttribute("xmlns");
			combined.removeAttribute("authorization");
			
			org.livee.data.tools.ProfileSchemaProcessor schema=org.livee.data.tools.ProfileSchemaProcessor.getInstance();
			String[] schema_warnings=schema.validateAttribute(combined);
			if(schema_warnings==null){
				SetProfileCommand command=CommandFactory.createSetProfileCommand(combined, passwd);
			
				DatabaseAdapter db = DatabaseAdapter.getInstance();
				db.postCommand(command);

				if (command.succeeded()){
					LogManager.postNormal("service.ProfileManagement200703.setProfile.end",null);
					return "<true />";
				}
			}else{
				// TODO Processing of Schema Warnings 

				
				LogManager.postError("service.ProfileManagement200703.setProfile.error","profile schema verification error");
				return "<warning />";
			}
		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.ProfileManagement200703.setProfile.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.ProfileManagement200703.setProfile.error", e.getMessage());
			return exp.getMessage();
		}

		// return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.ProfileManagement200703.setProfile.error", "unknown");
		return exp.getMessage();
		
	}

	public String updatePassword(String id, String old_passwd, String new_passwd) {
		
		LogManager.postNormal("service.ProfileManagement200703.updatePassword.begin",null);

		try {
			DatabaseAdapter db = DatabaseAdapter.getInstance();

			UpdatePasswordCommand command = new UpdatePasswordCommand(id,Base64Coder.decodeString(old_passwd), Base64Coder.decodeString(new_passwd));

			db.postCommand(command);

			if (command.succeeded()){
				LogManager.postNormal("service.ProfileManagement200703.updatePassword.end",null);
				return "<true />";
			}
		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.ProfileManagement200703.updatePassword.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.ProfileManagement200703.updatePassword.error", e.getMessage());
			return exp.getMessage();
		}

		// return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.ProfileManagement200703.updatePassword.error", "unknown");
		return exp.getMessage();
	}

	/* <error type="reason"/> */

}
