package org.livee.data.tools;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.livee.data.exception.LiveEException;
import org.livee.data.expr.AbstractData;
import org.livee.data.expr.AdminNeighborData;
import org.livee.data.expr.ParserFactory;
import org.livee.data.expr.DataFactory;

import org.livee.data.db.DatabaseAdapter;
import org.livee.data.db.GetAdminNeighborCommand;
import org.livee.data.db.SetAdminNeighborCommand;
import org.livee.util.Xml;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * 
 * @author Hideya Ochiai, The University of Tokyo
 */
public class NeighborManager extends Thread {

	private static NeighborManager m_Singleton=null;
	
	private int m_LinkStatusCheckerInterval;
	private int m_SharedTopologyLoaderInterval;
	private String m_ThisSite;
	private String m_Mode;  // active, passive
							// active mode = actively checks linkStatus and save the topology information into DB.
							// passive mode = passively load from DB and caches it. 
	
	private AbstractData m_AdminNeighbor;
	private AbstractData m_AdminParentSite;
	private AbstractData m_AdminColleagueSite;
	private AbstractData[] m_AdminChildSiteCol;
	
	private NeighborManager(){
		m_Singleton=this;
		m_Mode="passive";
		m_ThisSite=null;
		
		org.livee.data.tools.ConfigManager config=org.livee.data.tools.ConfigManager.getInstance();
		m_LinkStatusCheckerInterval=config.getIntParameter("/c:liveeConfiguration/c:neighbor/c:linkStatusChecker/c:interval/text()");
		m_SharedTopologyLoaderInterval=config.getIntParameter("/c:liveeConfiguration/c:neighbor/c:sharedTopologyLoader/c:interval/text()");
		
		if(m_LinkStatusCheckerInterval<=0){
			m_LinkStatusCheckerInterval=300;
		}
		if(m_SharedTopologyLoaderInterval<=0){
			m_SharedTopologyLoaderInterval=300;
		}
		sharedTopologyLoader();
		start();
	}
	
	public static NeighborManager getInstance(){
		if(m_Singleton==null){
			new NeighborManager();
		}
		return m_Singleton;
	}
	
	public void run(){
		while(true){
			try{
				if(m_Mode.equals("active")){
					linkStatusChecker();
					try{ Thread.sleep(m_LinkStatusCheckerInterval*1000); }catch(Exception e){}
				}else if(m_Mode.equals("passive")){
					sharedTopologyLoader();
					try{ Thread.sleep(m_SharedTopologyLoaderInterval*1000); }catch(Exception e){}	
				}
			}catch(LiveEException e){
				e.printStackTrace();
			}
		}
	}
	
	private void linkStatusChecker() throws LiveEException {
	
		LogManager.postNormal("neighbor.linkStatusChecker.begin", null);
		
		// copy m_AdminNeighbor ==> new_AdminNeighbor
		AbstractData new_AdminNeighbor=DataFactory.getInstance().create("adminNeighbor");
		java.util.Collection adminSiteCol=m_AdminNeighbor.getChildren();
		if(adminSiteCol!=null){
			java.util.Iterator itr=adminSiteCol.iterator();
			while(itr.hasNext()){
				AbstractData adminSite=(AbstractData)itr.next();
				AbstractData new_AdminSite=DataFactory.getInstance().create("adminSite");
				new_AdminNeighbor.append(new_AdminSite);
				String name=adminSite.getAttribute("name");
				String type=adminSite.getAttribute("type");
				new_AdminSite.setAttribute("name", name);
				new_AdminSite.setAttribute("type", type);
				
				java.util.Collection adminServerCol=adminSite.getChildren();
				if(adminServerCol!=null){
					java.util.Iterator itr2=adminServerCol.iterator();
					while(itr2.hasNext()){
						AbstractData adminServer=(AbstractData)itr2.next();
						AbstractData new_AdminServer=DataFactory.getInstance().create("adminServer");
						new_AdminSite.append(new_AdminServer);
						String service=adminServer.getAttribute("service");
						String url=adminServer.getAttribute("url");
						String linkStatus=adminServer.getAttribute("linkStatus");
						new_AdminServer.setAttribute("service", service);
						new_AdminServer.setAttribute("url", url);
						new_AdminServer.setAttribute("linkStatus", linkStatus);
					}
				}
			}
		}
		
		
		// linkStatusCheck
		boolean bAdminNeighborUpdated=false;
		java.util.Iterator itr=new_AdminNeighbor.getChildren().iterator();
		while(itr.hasNext()){
			AbstractData adminSite=(AbstractData)itr.next();
			String site=adminSite.getAttribute("name");
			String type=adminSite.getAttribute("type");

			if(type.equals("parent")){
				java.util.Iterator itr2=adminSite.getChildren().iterator();
				while(itr2.hasNext()){
					AbstractData adminServer=(AbstractData)itr2.next();
					String service=adminServer.getAttribute("service");
					String url=adminServer.getAttribute("url");
					String linkStatus=adminServer.getAttribute("linkStatus");
						
					if(service.equals("Admin200703")){
						String neighbor=null;
						boolean linkAlive=false;
						try{
							org.livee.data.soap.client.Admin200703Service srv=new org.livee.data.soap.client.Admin200703ServiceLocator();
							org.livee.data.soap.client.Admin200703_PortType admin=srv.getAdmin200703(new java.net.URL(url));
							neighbor=admin.getNeighbor();
							
							linkAlive=true;
						}
						catch(Exception e){
							linkAlive=false;
						}
						// Link information update
						if(linkAlive && (linkStatus==null || !linkStatus.equals("up"))){
							adminServer.setAttribute("linkStatus","up");
							bAdminNeighborUpdated=true;
						}
						if(!linkAlive && (linkStatus==null || !linkStatus.equals("down"))){
							adminServer.setAttribute("linkStatus", "down");
							bAdminNeighborUpdated=true;
						}

						if(neighbor!=null){
							try{
								Document doc=Xml.parseXml(neighbor);
								Element xml=doc.getDocumentElement();
								ParserFactory pf=ParserFactory.getInstance();
								AbstractData adminNeighbor=pf.get("adminNeighbor").parse(xml);
								int c=AdminNeighborData.getRelation(m_AdminNeighbor,adminNeighbor);
								if(c!=AdminNeighborData.NEIGHBOR_PARENT){
									LogManager.postWarning("neighbor.linkStatusChecker.warning", "Inconsistency neighbor site: "+site);
								}
							}catch(Exception e){
								e.printStackTrace();
							}
						}
					}
				}
				
			}else if(type.equals("child")){
				java.util.Iterator itr2=adminSite.getChildren().iterator();
				while(itr2.hasNext()){
					AbstractData adminServer=(AbstractData)itr2.next();
					String service=adminServer.getAttribute("service");
					String url=adminServer.getAttribute("url");
					String linkStatus=adminServer.getAttribute("linkStatus");
						
					if(service.equals("Admin200703")){
						
						String neighbor=null;
						boolean linkAlive=false;
						try{
							org.livee.data.soap.client.Admin200703Service srv=new org.livee.data.soap.client.Admin200703ServiceLocator();
							org.livee.data.soap.client.Admin200703_PortType admin=srv.getAdmin200703(new java.net.URL(url));
							neighbor=admin.getNeighbor();
							
							linkAlive=true;
						}
						catch(Exception e){
							linkAlive=false;
						}
						// Link information update
						if(linkAlive && (linkStatus==null || !linkStatus.equals("up"))){
							adminServer.setAttribute("linkStatus","up");
							bAdminNeighborUpdated=true;
						}
						if(!linkAlive && (linkStatus==null || !linkStatus.equals("down"))){
							adminServer.setAttribute("linkStatus", "down");
							bAdminNeighborUpdated=true;
						}

						if(neighbor!=null){
							try{
								Document doc=Xml.parseXml(neighbor);
								Element xml=doc.getDocumentElement();							
								ParserFactory pf=ParserFactory.getInstance();
								AbstractData adminNeighbor=pf.get("adminNeighbor").parse(xml);
								int c=AdminNeighborData.getRelation(m_AdminNeighbor,adminNeighbor);
								if(c!=AdminNeighborData.NEIGHBOR_CHILD){
									LogManager.postWarning("neighbor.linkStatusChecker.warning", "Inconsistency neighbor site: "+site);
								}
							}
							catch(Exception e){
								e.printStackTrace();
							}
						}

					}
				}
			}
			
			
		}

		if(bAdminNeighborUpdated){
			LogManager.postNormal("neighbor.linkStatusChecker.message", "Link Status Update Detected...");
			synchronized(m_Mode){	
				if(m_Mode.equals("active")){
					// 	save into db & parse neighbor 
					m_AdminNeighbor=new_AdminNeighbor;
					parseAdminNeighbor(m_AdminNeighbor);
					saveAdminNeighborIntoDB(m_AdminNeighbor);
				}
			}
		}

		LogManager.postNormal("neighbor.linkStatusChecker.end", null);
		
	}
	
	private void sharedTopologyLoader(){

		LogManager.postNormal("neighbor.sharedTopologyLoader.begin", null);

		synchronized(m_Mode){
			if(m_Mode.equals("passive")){
				// load from db
				AbstractData neighbor=loadAdminNeighborFromDB();
				m_AdminNeighbor=neighbor;
				parseAdminNeighbor(neighbor);
			}
		}
		
		LogManager.postNormal("neighbor.sharedTopologyLoader.end", null);
		
	}
	
	private void parseAdminNeighbor(AbstractData neighbor){
		
		java.util.Collection collection=neighbor.getChildren();
		java.util.ArrayList childSite=new java.util.ArrayList();
		
		boolean bParentUpdated=false;
		boolean bColleagueUpdated=false;
		java.util.Iterator itr=collection.iterator();
		while(itr.hasNext()){
			AbstractData adminSite=(AbstractData)itr.next();
			//String name=adminSite.getAttribute("site");
			String type=adminSite.getAttribute("type");
			
			if(type.equals("child")){
				childSite.add(adminSite);
			}else if(type.equals("parent")){
				m_AdminParentSite=adminSite;
				bParentUpdated=true;
			}else if(type.equals("colleague")){
				m_AdminColleagueSite=adminSite;
				m_ThisSite=adminSite.getAttribute("name");
				bColleagueUpdated=true;
			}
		}
		
		if(!bParentUpdated){
			m_AdminParentSite=null;
		}
		if(!bColleagueUpdated){
			m_AdminColleagueSite=null;
		}
		
		Object[] objArray=childSite.toArray();
		if(objArray!=null && objArray.length>0){
			AbstractData[] dataArray=new AbstractData[objArray.length];
			for(int i=0;i<objArray.length;i++){
				dataArray[i]=(AbstractData)objArray[i];
			}
			m_AdminChildSiteCol=dataArray;
		}else{
			m_AdminChildSiteCol=null;
		}	
	}
	
	private AbstractData loadAdminNeighborFromDB(){
		
		try{
			DatabaseAdapter db=DatabaseAdapter.getInstance();
			GetAdminNeighborCommand command=new GetAdminNeighborCommand();
			db.postCommand(command);
			
			if(command.succeeded()){
				return command.getAdminNeighbor();
			}
		}catch(Exception e){
			e.printStackTrace();
		}
		return null;
	}
	
	private void saveAdminNeighborIntoDB(AbstractData neighbor){
		try{
			DatabaseAdapter db=DatabaseAdapter.getInstance();
			SetAdminNeighborCommand command=new SetAdminNeighborCommand(neighbor);
			db.postCommand(command);
			
			if(command.succeeded()){
				return ;
			}
		}catch(Exception e){
			e.printStackTrace();
		}
		return ;		
	}
	
	public String getThisSiteName(){
		return m_ThisSite;
	}
	
	public AbstractData getParentAdminSite(){
		return m_AdminParentSite;
	}
	
	public AbstractData getColeagueAdminSite(){
		return m_AdminColleagueSite;
	}
	
	public AbstractData[] getChildrenAdminSite(){
		return m_AdminChildSiteCol;
	}

	public AbstractData getNeighbor(){
		return m_AdminNeighbor;		
	}
	
	public void setNeighbor(AbstractData neighbor){
		synchronized(m_Mode){
			m_Mode="active";
			m_AdminNeighbor=neighbor;
			parseAdminNeighbor(neighbor);
			saveAdminNeighborIntoDB(neighbor);
		}		
	}
}
