package org.livee.test;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.livee.data.db.CommandFactory;
import org.livee.data.db.DatabaseAdapter;
import org.livee.data.db.UploadCombinedCommand;
import org.livee.data.expr.AbstractData;
import org.livee.data.expr.DataFactory;

/**
 * 
 * @author Hideya Ochiai, The University of Tokyo
 */
public class SensorPool {
	
	private static java.util.Random m_Random=new java.util.Random();

	public static void main(String[] args) {

		int start=0;
		int end=99;
		
		try{
			start=Integer.parseInt(args[0]);
			end=Integer.parseInt(args[1]);			
		}
		catch(Exception e){
			System.out.println("Usage: $java org.livee.test.SensorPool start end");
			// System.out.println(" Default start=0");
			// System.out.println(" Default end=100");
			System.exit(1);
		}
		System.out.println("start="+start+"; end="+end);
		
		
		GenerateSensorID generator=new GenerateSensorID();
		String[] combined=generator.generateCombinedID(start, end);
		
		
		while(true){
			try{
				for(int i=0;i<combined.length;i++){
					DataFactory df=DataFactory.getInstance();
					AbstractData combinedSensor=df.create("combined");
					
					combinedSensor.setAttribute("id", combined[i]);
					
					String[] element=generator.generateElementID(combined[i]);
					String[] sensorType=generator.getSensorType();
					
					for(int k=0;k<element.length;k++){
						AbstractData elementSensor=df.create("element");
						elementSensor.setAttribute("id",element[k]);
						combinedSensor.append(elementSensor);
						
						AbstractData value=generateValue(sensorType[k]);
						elementSensor.append(value);
					}
					
					UploadCombinedCommand command=CommandFactory.createUploadCombinedCommand(combinedSensor, "00000000");
					DatabaseAdapter db=DatabaseAdapter.getInstance();
					
					try{
						db.postCommand(command);
						if(command.succeeded()){
							System.out.println("@"+i);
						}
					}
					catch(Exception e){
						e.printStackTrace();
					}
				}	
			}
			catch(Exception e){
				
			}

			try{
				Thread.sleep(60000);
			}
			catch(Exception e){}
		}
			
		
	}

	
	private static AbstractData generateValue(String sensorType){
		
		AbstractData value=DataFactory.getInstance().create("value");
		
		java.util.Calendar cal=java.util.Calendar.getInstance();
		String time=org.livee.util.W3CTimestamp.toString(cal);
		
		value.setAttribute("time",time);
		
		if(sensorType.equals("Temperature")){
			value.setTextContent(Double.toString(m_Random.nextDouble()*10+20));
		}else if(sensorType.equals("Humidity")){
			value.setTextContent(Double.toString(m_Random.nextDouble()*30+40));
		}else if(sensorType.equals("Pressure")){
			value.setTextContent(Double.toString(m_Random.nextDouble()*30+1000));
		}else if(sensorType.equals("RainFall")){
			value.setTextContent(Double.toString(m_Random.nextDouble()*3+0));
		}else if(sensorType.equals("DayRainFall")){
			value.setTextContent(Double.toString(m_Random.nextDouble()*50+0));
		}else if(sensorType.equals("WindDir")){
			value.setTextContent(Double.toString(m_Random.nextDouble()*360+0));
		}else if(sensorType.equals("WindSpeed")){
			value.setTextContent(Double.toString(m_Random.nextDouble()*20+0));
		}
		
		return value;
	}
}
